"""Console scripts.

Argument parsing for the console use of wurp."""

from __future__ import print_function

# Standard module
import argparse

import wurp

manual = {
          'cal-tc': 'Calculate transport coefficients from energy flux data.',

          'wurp': ("CURP - CURrent calculation for Proteins.\n"
                   "Compute inter-residue flow of energy or heat and atomic"
                   "stress tensors in a protein. Create Inter-residue Energy"
                   "Exchange Network as a graph.\n\n"
                   "This package also contains functionalities to format "
                   "the trajectories and analyze said results.\n\n"
                   "Developed by Yamato's Laboratory, University of Nagoya, "
                   "Japan."),

          'compute': ('Launch flux or stress tensor computations given a '
                      'configuration file.'),

          'graph-een': 'Show network chart of the energy conductivities.',

          'sum-acf': ('Average auto-correlation function over the given '
                      'trajectories'),
        }


def arg_wurp():
    """Parse command line arguments of wurp command."""

    parser = argparse.ArgumentParser(description=manual['wurp'],
                                     prog='wurp')

    # Add commands as subparsers.
    sp = parser.add_subparsers(dest='command', help='Wurp commands')
    sp_compute = sp.add_parser('compute',
                               help=manual['compute'])
    sp_cal_tc = sp.add_parser('cal-tc',
                              help=manual['cal-tc'])
    sp_graph_een = sp.add_parser('graph-een',
                                 help=manual['graph-een'])
    sp_sum_acf = sp.add_parser('sum-acf',
                               help=manual['sum-acf'])

    from wurp.script.console import manual as script_manual
    sp_conv_trj = sp.add_parser('conv-trj',
                                help=script_manual['conv-trj'])
    sp_analyze = sp.add_parser('analyze',
                               help=script_manual['analyze'])

    from wurp.tool.console import manual as tool_manual
    sp_tool = sp.add_parser('tool',
                            help=tool_manual['tool'])

    # Add arguments and subparsers to each command.
    arg_compute(sp_compute)
    arg_cal_tc(sp_cal_tc)
    arg_graph_een(sp_graph_een)
    arg_sum_acf(sp_sum_acf)

    from wurp.script.console import arg_conv_trj, arg_analyze
    arg_conv_trj(sp_conv_trj)
    arg_analyze(sp_analyze)

    from wurp.tool.console import arg_tool
    arg_tool(sp_tool)

    return(parser)


def exec_command(parser):
    """Call scripts depending on command line argument.

    Parameters
    ----------
    parser : argparse.ArgumentParser
    """

    args = parser.parse_args()
    command = args.func
    command(**vars(args))


def arg_compute(parser=None):
    """Parse command line arguments of wurp compute"""

    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['compute']
    parser.prog = 'compute'

    # Set the default function used by the parser.
    parser.set_defaults(func=wurp.wurp)

    # Definitions
    parser.add_argument('-v', '--vervose',
                        dest='vervose', default=False,
                        action='store_true',
                        help='Print out informations.')

    parser.add_argument('-s', '--enable-serial',
                        dest='use_serial', default=False,
                        action='store_true',
                        help=("Calculate in serial, don't calculate"
                              "in parallel."))

    parser.add_argument('--output-conf-default',
                        dest='output_conf_def', default=False,
                        action='store_true',
                        help=('Output the config parameters in ini format '
                              'with default values.'))

    parser.add_argument('--output-conf-formatted',
                        dest='output_conf_fmtd', default=False,
                        action='store_true',
                        help=('Output the config parameters in rest style '
                              'with default values.'))

    parser.add_argument('input_', nargs='?',
                        default='run.cfg',
                        help='Specify input filenames.')

    if return_parser:
        return parser


def arg_cal_tc(parser=None):
    """Parse the command line arguments of wurp cal-tc."""

    # Make argument parser
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['cal-tc']
    parser.prog = 'cal-tc'
    parser.set_defaults(func=wurp.script.cal_tc)

    # add argument definitions
    parser.add_argument(dest='flux_fn', metavar='FLUX_FILENAME',
                        help=('The filepath of flux data.'))

    parser.add_argument('-o', '--tc-file', metavar='FILE',
                        dest='tc_fn', required=True,
                        help=('The filename of tc data.'))

    parser.add_argument('-a', '--acf-file', metavar='FILE',
                        dest='acf_fn', required=False,
                        default='',
                        help=('The filename of acf data.'))

    parser.add_argument('-af', '--acf-file-format', metavar='FORMAT',
                        dest='acf_fmt', required=False,
                        default="netcdf", choices=['ascii', 'netcdf'],
                        help=('The file format of acf data.'))

    parser.add_argument('-r', '--frame-range',
                        metavar=('FIRST', 'LAST', 'INTER'),
                        dest='frame_range', required=False,
                        default=[1, -1, 1], type=int, nargs=3,
                        help='The range of frame; first frame, last frame, '
                              'interval step.')

    parser.add_argument('-s', '--average-shift', metavar='SHIFT',
                        dest='avg_shift', required=False,
                        default=1, type=int,
                        help='The frame to shift for averaging.')

    parser.add_argument('--sample-number', dest='nsample',
                        type=int, required=False, default=0,
                        help=('number of sample for one flux data file. '
                              'Default value is 0 that present to make samples'
                              ' as much as possible'))

    parser.add_argument(
        '-dt', '--dt', metavar='DT', dest='d_t', required=False,
        default=None, type=float,
        help=('t of between the neighbour frames. The unit is in ps. '
              + 'Default value is determined by time variable in flux data.'))

    parser.add_argument('-c', '--coefficient', metavar='COEFFICIENT',
                        dest='coef', required=False,
                        default=1.0, type=float,
                        help='Multiply acf by given coefficient.')

    parser.add_argument('-v', '--vervose', dest='use_debug',
                        action='store_true', required=False,
                        help='turn on debug mode.')

    parser.add_argument('--no-axes', dest='no_axes',
                        action='store_true', required=False,
                        help='with this option, scalar flux is handled.')

    if return_parser:
        return parser


def arg_graph_een(parser=None):
    """Parse the command line arguments of wurp graph-een."""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['graph-een']
    parser.proc = 'graph-een'
    parser.set_defaults(func=wurp.script.graph_een)

    # Add argument definitions
    parser.add_argument(
            'data_fns', nargs='+',
            help='Specify filenames. ')
    parser.add_argument(
            '-t', '--targets', dest='targets', required=False,
            nargs='*', default=['1-'],
            help=('This option allow you to show only target '
                  'and thier neighbhor nodes. '
                  'ex), [-t 1-5], [-t 2-9 15 17], [-t 25-].'))
    parser.add_argument(
            '--forced-output-nodes', dest='force_nodes', required=False,
            nargs='*', default=[],
            help=('nodes to forcibly show.'
                  'ex), [-t 1-5], [-t 2-9 15 17], [-t 25-].'))
    parser.add_argument(
            '-p', '--bring-node-pair-close-together', dest='close_pairs',
            required=False, nargs='*', default=[],
            help=('bring the node pair close together on the EEN graph.'
                  'ex), 1:2, 3:5, 3:15.'))
    parser.add_argument(
            '-c', '--cluster-filename', dest='cluster_fn',
            required=False, default='',
            help=('Specify a cluster file name. '))
    parser.add_argument('-n', '--node-style-filename', dest='node_fn',
                        required=False, default='',
                        help=('Specify a file name which node '
                              'style difinitions are written.')
                        )
    parser.add_argument(
            '--with-one-letter', dest='use_1letter', default=False,
            required=False, action='store_true',
            help='use 1 letter representation for amino acid name.')
    parser.add_argument(
            '-f', '--output-een-filename',
            dest='fig_fn', required=False,
            help='Specify a filename to graph EEN.')
    parser.add_argument(
            '-r', '--threshold', dest='threshold',
            type=float, required=False, default=None,
            help='threshold value to show nodes on chart figure.')
    parser.add_argument(
            '-R', '--topology-threshold', dest='tpl_threshold',
            type=float, required=False, default=None,
            help=("threshold value to define graph topology. "
                  "If you don't give this parameter, "
                  "the value of threshold to show nodes will be used."))

    # Esthetic parameters
    parser.add_argument(
            '--ratio', dest='ratio',
            type=float, required=False, default=None,
            help=("ratio of height/width to chart a figure. "
                  "If '--graph-size is set, this option will be overwritten"
                  "by it."))
    parser.add_argument(
            "-s", "--graph-size", dest="graph_size",
            required=False, default=None,
            help=("The graph size in inch unit. ex) -s '3.4,2.5'"))
    parser.add_argument(
            '--title', dest='title', required=False, default='',
            help='Specify the title of the figure.')
    parser.add_argument(
            '--direction', dest='direction',
            required=False, choices=['LR', 'TB'], default='TB',
            help='Specify the kind of the direction.')
    parser.add_argument(
            '-I', '--hide-isolated-nodes', dest='hide_isonode',
            required=False, action='store_true',
            help='Hide isolated nodes when appling multiple irEC files.')
    parser.add_argument(
            '--show-negative-values', dest='use_decrease',
            action='store_true',
            help="show the nodes pair which have negative values.")
    parser.add_argument(
            '--alpha', dest='alpha',
            type=int, required=False, default=None,
            help='The transparency value of the background color.')
    parser.add_argument(
            '-lv', '--line-values', dest='line_values', required=False,
            nargs='*', type=float, default=[0.015, 0.008, 0.003],
            help=('The threshold values for line attributes. '
                  'Number of elements in the list must be equal '
                  'with all line attribute.'))
    parser.add_argument(
            '-lc', '--line-colors', dest='line_colors', required=False,
            nargs='*', type=str, default=['red', 'blue', 'green'],
            help=('The colors of line. Number of elements in the list '
                  'must be equal with all line attribute.'))
    parser.add_argument(
            '-lt', '--line-thicks', dest='line_thicks', required=False,
            nargs='*', type=float, default=[4.0, 4.0, 2.5],
            help=('The thickness of line. Number of elements in the list '
                  'must be equal with all line attribute.'))
    parser.add_argument(
            '-lw', '--line-weights', dest='line_weights', required=False,
            nargs='*', type=float, default=[5.0, 3.0, 1.0],
            help=('The weight of line. Number of elements in the list must'
                  'be equal with all line attribute.'))

    if return_parser:
        return parser


def arg_sum_acf(parser=None):
    """Parse command line arguments of wurp sum-acf"""

    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['sum-acf']
    parser.prog = 'sum-acf'

    # Set the default function used by the parser.
    parser.set_defaults(func=wurp.script.sum_acf)

    # Definitions
    parser.add_argument(
            dest='acf_fps', metavar='ACF_FILE', nargs='+',
            help=('The filepath of auto-correlation function data.'))
    parser.add_argument('-a', '--acf-file', metavar='FILE',
                        dest='output_acf_fp', required=True,
                        default='',
                        help=('The filepath of acf data.'))
    parser.add_argument('-t', '--tcs-file', metavar='FILE',
                        dest='tcs_fp', required=False,
                        default='',
                        help=('The filepath of tc time series data.'))
    parser.add_argument('-c', '--coefficient', metavar='COEFFICIENT',
                        dest='coef', required=False,
                        default=1.0, type=float,
                        help='Multiply acf by given coefficient.')

    if return_parser:
        return parser


def main():
    parser = arg_wurp()
    exec_command(parser)


if __name__ == '__main__':
    main()



#! /usr/bin/env python2
from __future__ import print_function

import os
import sys


def parse_ss(filename):
    """
    Parse the secondary structure file that is as follows:

    %TM1  1 25
    %TM2  40 90
    %SHEET1 102 110
    .
    .
    .
    """

    file = open(filename, 'rb')
    lines = (line for line in file
            if line.startswith("%H")
            or line.startswith("%TM")
            or line.startswith("%S"))

            # if line.startswith("%TM")
            # if line.startswith("%H") )

    for line in lines:
        cols = line.split()
        name, ibeg, iend = cols[0].replace('%',''), cols[1], cols[2]
        yield int(ibeg), int(iend)

    file.close()


def is_loop(rid, rname, ibeg_iend_pairs, excluded_list):
    for ibeg, iend in ibeg_iend_pairs:
        if ibeg <= rid <= iend:
            return False
    else:
        return not include_resname(rname, excluded_list)


def include_resname(resname, rname_list):
    rname = resname.lower()
    for cs in rname_list:
        if rname.startswith(cs.lower()):
            return True
    else:
        return False


def main(ec_fn, ss_fn, excluded_list=['WAT'], **kwds):
    ibeg_iend_pairs = list(parse_ss(ss_fn))

    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
    lines = (line for line in ec_file
             if not line.startswith('#')
             if not line.isspace() )
    ec_file.close()

    for line in lines:
        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]
        rid1 = int(don_line.split('_')[0])
        rid2 = int(acc_line.split('_')[0])
        rname1 = don_line.split('_')[1]
        rname2 = acc_line.split('_')[1]

        if is_loop(rid1, rname1, ibeg_iend_pairs, excluded_list) \
                or is_loop(rid2, rname2, ibeg_iend_pairs, excluded_list):
            continue

        else:
            print(line.strip())

if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_noloop
    parser = arg_sel_noloop()
    exec_command(parser)



#! /usr/bin/env python
from __future__ import print_function
import os, sys
import math
import numpy
import gzip

# fortran module
from wurp.script.lib_acf import cal_acf

# Exception
class NumFrameError(Exception): pass

def parse_options():
    """Parse and get the command line options."""

    # make argument parser
    import argparse
    parser = argparse.ArgumentParser(
        description= ('Calculate auto-correlation function data '
            'from energy flux datas.') )

    # add argument definitions
    parser.add_argument(
            'flux_fns', nargs='+',
            help=('specify filenames of flux data. '
                'ex.) flux.dat0001, flux.dat0002, ...') )

    parser.add_argument(
            '-a', '--output-acf', dest='acf_fn', required=False, default='',
            help='specify a filename to output acf.')

    parser.add_argument(
            '-D', '--without-dispersion', dest='enable_disp',
            action='store_false', required=False,
            help='calculate dispersion for tc.')

    parser.add_argument(
            '-t', '--output-tcs', dest='tcs_fn', required=False, default='',
            help=('specify a filename to output'
                  'time series of transport coefficient.'))

    # parser.add_argument(
            # '-p', '--with-precise', dest='enable_precise',
            # action='store_true', required=False,
            # help='calculate dispersion in precise.')

    parser.add_argument(
            '-f', '--frame-range', dest='frame_range',
            type=int, nargs=3, required=True,
            help='frame range; first_frame, last_frame, intervalstep.')

    parser.add_argument(
            '-s', '--shift', dest='frm_shift',
            type=int, required=False, default=1,
            help='shift frame to collect samples.')

    parser.add_argument(
            '--sample-number', dest='nsample',
            type=int, required=False, default=0,
            help=('The number of samples for one flux data file. '
                'Default value is 0 that present to make samples '
                'as much as possible'))

    parser.add_argument(
            '-n', '--with-norm', dest='enable_norm',
            action='store_true', required=False,
            help='flag to normalize acf.')

    parser.add_argument(
            '-dt', '--dt', dest='dt',
            type=float, required=False, default=1.0,
            help='t of between the neighbour frames. The unit is in ps')

    parser.add_argument(
            '-c', '--coefficient', dest='coef',
            type=float, required=False, default=1.0,
            help='write coefficient times acf.')

    parser.add_argument(
            '-o', '--output-log', dest='log_fn', required=False,
            default='tc.log',
            help='specify a filename for log.')

    parser.add_argument(
            '-v', '--vervose', dest='use_debug',
            action='store_true', required=False,
            help='turn on debug mode.')

    # make arguments
    options = parser.parse_args()

    return options

def load_fluxdata(flux_fn):

    if flux_fn.endswith('.gz'):
        file = gzip.open(flux_fn, 'rb')
    else:
        file = open(flux_fn, 'rb')

    ifrm = 0
    xss = []
    for line in file:
        if line.startswith('#'): continue

        cols = line.split()
        ifrm += 1
        xss.append( float(cols[1]) )

    file.close()
    return numpy.array(xss)

def cal_tc(acf, dt, coef=1.0):
    """Calculate final transport  coefficient."""
    return coef * dt*1000.0 * numpy.sum(acf)

def cal_tcs(acf, dt, coef=1.0):
    """Calculate time series of transport coefficient."""
    return coef * dt*1000.0 * numpy.add.accumulate(acf)

def write_acf(times, acf, opts):
    """Write auto-correlation function data to given file."""

    # open file
    acf_fn = opts.acf_fn
    if acf_fn.endswith('.gz'):
        file = gzip.open(acf_fn, 'wb')
    else:
        file = open(acf_fn, 'wb')

    # get donor and acceptor id and name
    flux_fn = opts.flux_fns[0]
    don_acc_line = os.path.basename(flux_fn).split('.')[0]
    don_id, don_name, acc_id, acc_name = don_acc_line.split('_')
    don_id, acc_id = int(don_id), int(acc_id)

    # write header
    file.write('#donor    {}{}\n'.format(don_name, don_id))
    file.write('#acceptor {}{}\n'.format(acc_name, acc_id))
    file.write('#label {:>8}  {:>22}\n'.format('time(ps)','acf(kcal/mol/fs)^2'))

    # write data
    for time, a in zip(times, acf[:, 0]):
        file.write("{:15.4f}  {:22.14e}\n".format(time, a))

    # close acf data file
    file.close()

def write_tc(tc, tc_disp, don_acc_line):
    """Write transport coefficient and dispersion to standard output."""
    if tc_disp:
        print( don_acc_line, tc, '+-', tc_disp)
    else:
        print( don_acc_line, tc)

def get_don_acc_line(flux_fn):
    don_acc_line = os.path.basename(flux_fn).split('.')[0]
    don_id, don_name, acc_id, acc_name = don_acc_line.split('_')
    return "{}_{} {}_{}".format( don_id, don_name, acc_id, acc_name)

def write_tc_traj(itrj, tc, don_acc_line):
    """Write transport coefficient and dispersion to standard output."""


def write_acf(times, acf, acf_fn, flux_fn):

    # open file
    if acf_fn.endswith('.gz'):
        file = gzip.open(acf_fn, 'wb')
    else:
        file = open(acf_fn, 'wb')

    # get donor and acceptor id and name
    don_acc_line = os.path.basename(flux_fn).split('.')[0]
    don_id, don_name, acc_id, acc_name = don_acc_line.split('_')
    don_id, acc_id = int(don_id), int(acc_id)

    # write header
    file.write('#donor    {}{}\n'.format(don_name, don_id))
    file.write('#acceptor {}{}\n'.format(acc_name, acc_id))
    file.write('#label {:>8}  {:>22}\n'.format(
        'time(ps)', 'acf[(kcal/mol/fs)^2]'))

    # write data
    for time, a in zip(times, acf[:, 0]):
        file.write("{:15.4f}  {:22.14e}\n".format(time, a))

    # close tcs data file
    file.close()

def write_tcs(times, tcs, disps, tcs_fn, flux_fn):

    # open file
    if tcs_fn.endswith('.gz'):
        file = gzip.open(tcs_fn, 'wb')
    else:
        file = open(tcs_fn, 'wb')

    # get donor and acceptor id and name
    don_acc_line = os.path.basename(flux_fn).split('.')[0]
    don_id, don_name, acc_id, acc_name = don_acc_line.split('_')
    don_id, acc_id = int(don_id), int(acc_id)

    # write header
    file.write('#donor    {}{}\n'.format(don_name, don_id))
    file.write('#acceptor {}{}\n'.format(acc_name, acc_id))
    file.write('#label {:>8}  {:>22} {:>22}\n'.format(
        'time(ps)', 'tcs[(kcal/mol)^2/fs])', 'dispersion' ) )

    # write data
    for time, tc, disp in zip(times, tcs[:,0], disps[:,0]):
        file.write("{:15.4f}  {:22.14e} {:22.14e}\n".format(time, tc, disp))

    # close tcs data file
    file.close()

class Log:
    def __init__(self, filename):
        self.__fn = filename

        # initialize log file
        with open(filename, 'wb') as file:
            pass

    def write(self, line):
        with open(self.__fn, 'ab') as file:
            file.write(str(line) + '\n')

def sum_gen(gen):
    first = gen.next()
    for g in gen:
        first += g

    return first


def main():

    # parse command line options
    opts = parse_options()

    log = Log(opts.log_fn)
    if opts.use_debug:
        log.write(opts)

    frm_first, frm_last, frm_interval = opts.frame_range
    ncom = 1

    # prepare
    nfile = len(opts.flux_fns)
    nacf = (frm_last - frm_first)/frm_interval + 1

    # generator acf by data file
    def gen_acf(flux_fns, tag=''):
        for ifn_1, flux_fn in enumerate(flux_fns):
            fluxes = load_fluxdata(flux_fn)

            if len(fluxes) < nacf:
                raise NumFrameError(("\n"+
                    "   Number of flux data is {},\n" +
                    "   but given frame range is {}.")
                    .format(len(fluxes), nacf))

            acf = cal_acf(fluxes,nacf,frm_first,frm_last,frm_interval,
                    opts.frm_shift, norm=opts.enable_norm, nsample=opts.nsample)
            log.write('[{tag}] finished {number}'.
                    format(tag=tag, number=ifn_1+1))
            yield acf

    # set coefficient
    dt = opts.dt * opts.frame_range[2] # in ps
    times = numpy.arange(0.0, nacf*dt, dt)

    ################################
    # final tc and dispersion only #
    ################################
    don_acc_line = get_don_acc_line(opts.flux_fns[0])
    if (not opts.acf_fn) and opts.enable_disp:

        acf_iter = gen_acf(opts.flux_fns, 'tc+disp')
        tc_ary = numpy.array( [cal_tc(acf, dt, opts.coef) for acf in acf_iter] )
        for itraj, tc in enumerate(tc_ary): # by trajectory(file)
            print("%itraj="+str(itraj), don_acc_line, tc)

        # calculate tc and dispersion
        tc_avg  = numpy.sum(tc_ary) / nfile
        tc2_avg = numpy.sum(tc_ary*tc_ary) / nfile

        # average tc from tc array
        tc = tc_avg
        tc_disp = math.sqrt( tc2_avg - tc_avg**2 )
        print( don_acc_line, tc, '  ', tc_disp)

    ####################
    # acf and final tc #
    ####################
    elif opts.acf_fn and (not opts.enable_disp):
        acf_iter = gen_acf(opts.flux_fns, 'acf+tc')

        # get average acf and tc
        acf_avg = sum(acf_iter) / nfile
        # calculate tc from averaged acf
        tc = cal_tc(acf_avg, dt, opts.coef)

        write_acf(times, acf_avg, opts.acf_fn, opts.flux_fns[0])
        print( don_acc_line, tc)

    ##################################
    # acf and final tc +- dispersion #
    ##################################
    elif opts.acf_fn and opts.enable_disp:
        acf_iter = gen_acf(opts.flux_fns, 'acf+tc+disp')

        tc_list = []
        acf_sum = numpy.zeros([nacf,ncom])
        for acf in acf_iter:
            acf_sum += acf
            tc_list.append(cal_tc(acf, dt, opts.coef))

        # calculate tc and dispersion
        tc_ary  = numpy.array(tc_list)
        for itraj, tc in enumerate(tc_ary):
            print("%itraj="+str(itraj), don_acc_line, tc)

        tc_avg  = numpy.sum(tc_ary) / nfile
        tc2_avg = numpy.sum(tc_ary*tc_ary) / nfile

        # get average_acf and tc dispersion
        acf_avg = acf_sum / nfile
        tc      = cal_tc(acf_avg, dt, opts.coef)
        # tc_avg and tc muse be equal !
        tc_disp = math.sqrt( tc2_avg - tc_avg**2 )

        write_acf(times, acf, opts.acf_fn, opts.flux_fns[0])
        print( don_acc_line, tc, '  ', tc_disp)

    #################################################
    #  get tcs( time-series of tc ) and dispersions #
    #################################################
    if opts.tcs_fn: # and (not opts.enable_precise):
        acf_iter = gen_acf(opts.flux_fns, 'tcs')

        tcs_sum  = numpy.zeros([nacf,ncom])
        tcs_sum2 = numpy.zeros([nacf,ncom])
        for acf in acf_iter:
            tcs = cal_tcs(acf, dt, opts.coef)
            tcs_sum  += tcs
            tcs_sum2 += tcs * tcs

        tcs_avg  = tcs_sum / nfile
        tcs2_avg = tcs_sum2 / nfile

        # dispersion of tc based on <x^2> - <x>^2
        tcs_disp = numpy.sqrt( tcs2_avg - tcs_avg**2 )
        # tc_disp and tcs_disp[-1] must be equal !
        if opts.use_debug: log.write('tc = {:22.14e} {:22.14e}'
            .format(tc, tcs_avg[-1][0]))
        if opts.use_debug: log.write('disp = {:22.14e} {:22.14e}'
            .format(tc_disp, tcs_disp[-1][0]))

        write_tcs(times, tcs_avg, tcs_disp, opts.tcs_fn, opts.flux_fns[0])

    ############################################################
    #  get tcs( time-series of tc ) and dispersions in precise #
    ############################################################
    # if opts.tcs_fn and opts.enable_precise:
        # tcs_avg = cal_tcs(acf_avg, dt, opts.coef)

        # acf_iter = gen_acf(opts.flux_fns)

        # tcs2_sum  = numpy.zeros([nacf,ncom])

        # def gen_diff2(tcs_avg, acf_iter):
            # for acf in acf_iter:
                # tcs = cal_tcs(acf, dt)
                # yield (tcs - tcs_avg)**2

        # # dispersion of tc based on <(dx)^2>
        # tcs_disp = numpy.sqrt( sum(gen_diff2(tcs_avg, acf_iter)) / nfile )

        # write_tcs(times, tcs_avg, tcs_disp, opts.tcs_fn, opts.flux_fns[0])

    # write
    # write_tc(acf, opts)

if __name__ == '__main__':
    main()



"""Tool console scripts."""

from __future__ import print_function

# Standard module
import argparse
import wurp.tool.sele as sele
from wurp.console import exec_command


manual = {
          'sele': ('Output an irEC file with removed or kept only selected '
                   'residues from input irEC file.'),
          'helix': '',
          'neighbor': '',
          'noasa': '',
          'nocap': 'Remove residue pairs that contain the capped residues.',
          'nohelix': '',
          'noloop': '',
          'noneighbor': ('Remove residue pairs that indicate covalent '
                         'peptide bonds.'),
          'threshold': ''
          }


def arg_sele(parser=None):

    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['sele']
    parser.prog = 'sele'

    # Add commands as subparsers
    sp = parser.add_subparsers(dest='sele', help='Wurp tool select commands')

    cmd_list = manual.keys()
    cmd_list.remove('sele')
    cmd_sp = {cmd: sp.add_parser(cmd, help=manual[cmd]) for cmd in cmd_list}

    arg_sel_helix(cmd_sp['helix'])
    arg_sel_neighbor(cmd_sp['neighbor'])
    arg_sel_noasa(cmd_sp['noasa'])
    arg_sel_nocap(cmd_sp['nocap'])
    arg_sel_nohelix(cmd_sp['nohelix'])
    arg_sel_noloop(cmd_sp['noloop'])
    arg_sel_noneighbor(cmd_sp['noneighbor'])
    arg_sel_thresh(cmd_sp['threshold'])


    if return_parser:
        return parser


def arg_sel_helix(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['helix']
    parser.prog = 'sel_helix'
    parser.set_defaults(func=sele.helix)

    parser.add_argument('helix_fn', help='')
    parser.add_argument('arg_3', default=None)
    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser

def arg_sel_neighbor(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['neighbor']
    parser.prog = 'sel_neighbor'
    parser.set_defaults(func=sele.neighbor)

    parser.add_argument('excluded_list', default=['WAT'], nargs='+')
    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def arg_sel_noasa(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['noasa']
    parser.prog = 'sel_noasa'
    parser.set_defaults(func=sele.noasa)

    parser.add_argument('asa_fn')
    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def arg_sel_nocap(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['nocap']
    parser.prog = 'sel_nocap'
    parser.set_defaults(func=sele.nocap)

    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def arg_sel_nohelix(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['nohelix']
    parser.prog = 'sel_nohelix'
    parser.set_defaults(func=sele.nohelix)

    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))
    parser.add_argument('helix_fn')
    parser.add_argument('arg_3', default=None)

    if return_parser:
        return parser


def arg_sel_noloop(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['noloop']
    parser.prog = 'sel_noloop'
    parser.set_defaults(func=sele.noloop)

    parser.add_argument('ss_fn')
    parser.add_argument('excluded_list', nargs='+', default=['WAT'])
    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def arg_sel_noneighbor(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['noneighbor']
    parser.prog = 'sel_noneighbor'
    parser.set_defaults(func=sele.noneighbor)

    parser.add_argument('excluded_list', nargs='+', default=['WAT'])
    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def arg_sel_thresh(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['threshold']
    parser.prog = 'sel_thresh'
    parser.set_defaults(func=sele.threshold)

    parser.add_argument('-f','--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))


def main():
    parser = arg_sele()
    exec_command(parser)


if __name__ == '__main__':
    main()



# Getting main wurp function.
from .compute import wurp

# Writer
from .parser.writer import Writer as TrjWriter

import parallel
if parallel.use_mpi:
    ParallelProcessor = parallel.ParallelProcessor
else:
    ParallelProcessor = parallel.SequentialProcessor

# Topology
from .parser import get_tplprm_simple as get_tpl

# Trajectory
from .script.conv_trj import gen_trj

# Get all the nice scripts
from . import script



"""parallel - Classes to handle processor, parallel or serial

Classes:
ParallelProcessor -- Run the calculations in parallel.
SequentialProcess -- Run the calculations in serial.
"""

from __future__ import print_function

import itertools as it
import time

try:
    from mpi4py import MPI
    use_mpi = True
except ImportError:
    use_mpi = False

################################################################################
class ParallelProcessor:

    """
    The class to run in parallel from given method and dictionary list,
    using mpi4py library.

    >>> para = ParallelProcessor()
    >>> results = para.run(method, crd=crds, vel=vels)

    The calculated result return as generator.

    >>> for res in results:
        if para.is_root():
    ...     print(res)

    To perform test calculation, You can be run as the main module:

        $ mpiexec -n 8 python parallel.py

    """

    def __init__(self, chunksize=5):
        self.__chunksize = chunksize

        self.__comm = MPI.COMM_WORLD
        self.__nproc = self.__comm.Get_size()
        self.__iproc = self.__comm.Get_rank()

        self.__nstep = 0
        self.__other_time    = None
        self.__scatter_times = []
        self.__body_times    = []
        self.__gather_times  = []

    def is_root(self):
        return self.__iproc == 0

    def get_rank(self):
        return self.__iproc

    def get_nproc(self):
        return self.__nproc

    def _gen_kwds(self, kwds):
        """Generate each dictionary from listed dictionaries."""

        ks = kwds.keys()    # kwd1, kwd2, ...
        npair = len(ks)     # the number of kwds

        if self.is_root():
            vs = kwds.values()  # val1_iter, val2_iter, ...
            pair_vs_iter = it.izip(*vs)

        else:
            def gen_none():
                while True:
                    yield npair*[None]

            pair_vs_iter = gen_none()

        while True:

            is_end = False
            if self.is_root():
                try:
                    pair_vs = pair_vs_iter.next()
                except StopIteration:
                    is_end = True

            else:
                pair_vs = pair_vs_iter.next()

            is_end = self.__comm.bcast(is_end, root=0)
            
            if is_end:
                break

            # (val1, val2, ...)_1, (val1, val2, ...)_2, ...
            yield dict( (kwd, val) for kwd, val in zip(ks, pair_vs) )

    def _gen_kwds_nproc_list(self, kwds_iter):
        """Generate a list of the number of precesses for keyword dictionary."""
        iproc = 0
        kwds_list_sumup = []
        for kwds in kwds_iter:
            kwds_list_sumup.append(kwds)
            iproc += 1

            if iproc == self.__nproc:
                yield kwds_list_sumup
                kwds_list_sumup = []
                iproc = 0

        else:
            if len(kwds_list_sumup) != 0:
                for ip in range(self.__nproc-len(kwds_list_sumup)):
                    kwds_list_sumup.append(None)
                yield kwds_list_sumup

    def run(self, method, *args, **kwds):
        """Generate result after run in parallell."""

        scatter_times = []
        method_times  = []
        gather_times  = []
        other_time = 0.0

        kwds_nproc_iter = self._gen_kwds_nproc_list( self._gen_kwds(kwds) )

        t0 = time.time()

        istep = 0
        for istep, kwds_list in enumerate(kwds_nproc_iter):

            # scatter method's kwds on each process

            t1 = time.time()
            kwds = self.__comm.scatter(kwds_list, root=0)

            # run on each process
            t2 = time.time()
            if kwds:
                result = method(**kwds)
            else:
                result = None

            # gather return data from on each process
            t3 = time.time()
            if self.get_nproc() != 1:
                results = self.__comm.gather(result, root=0)
            else:
                results = [result]
            t4 = time.time()

            # split all results on each processes.
            if results:
                for res in results:
                    if res is not None:
                        yield res

            # store time intervals for each process
            scatter_times.append(t2-t1)
            method_times.append(t3-t2)
            gather_times.append(t4-t3)

            t5 = time.time()
            other_time += t5 - t4

        t6 = time.time()

        self.__mpi_time   = t6 - t0
        self.__other_time = other_time
        self.__nstep = istep
        self.__scatter_times = self.__comm.gather(scatter_times, root=0)
        self.__body_times  = self.__comm.gather(method_times, root=0)
        self.__gather_times  = self.__comm.gather(gather_times, root=0)

    def run_nogather(self, method, *args, **kwds):
        """Generate result after run in parallell."""

        scatter_times = []
        method_times  = []
        gather_times  = []

        kwds_nproc_iter = self._gen_kwds_nproc_list( self._gen_kwds(kwds) )

        t0 = time.time()

        istep = 0
        for istep, kwds_list in enumerate(kwds_nproc_iter):

            # scatter method's kwds on each process

            t1 = time.time()
            kwds = self.__comm.scatter(kwds_list, root=0)

            # run on each process
            t2 = time.time()
            if kwds:
                method(**kwds)

            # store time intervals for each process
            scatter_times.append(t2-t1)
            method_times.append(t3-t2)
            gather_times.append(0.0)

            # yield result
            yield None

        t5 = time.time()
        self.__mpi_time = t5 - t0
        self.__other_time = 0.0
        self.__nstep = istep
        self.__scatter_times = self.__comm.gather(scatter_times, root=0)
        self.__body_times  = self.__comm.gather(method_times, root=0)
        self.__gather_times  = self.__comm.gather(gather_times, root=0)

    def get_time_info(self, nstep):

        import numpy
        nproc = self.__nproc
        denom = 1.0/float(nproc*nstep)

        stime_total = sum([sum(times) for times in self.__scatter_times])
        btime_total = sum([sum(times) for times in self.__body_times   ])
        gtime_total = sum([sum(times) for times in self.__gather_times ])

        return [ ('Scatter', stime_total*denom),
                 ('Body'   , btime_total*denom),
                 ('Gather' , gtime_total*denom) ]

    def get_mpi_time(self):
        return self.__mpi_time

    def get_other_time(self):
        return self.__other_time

    def write(self, *args, **kwds):
        if self.is_root():
            print(*args, **kwds)

################################################################################
class SequentialProcessor:

    """Class to run in serial from given method and dictionary list,
    when there is not mpi4py library.

    >>> para = SequentialProcessor()
    >>> results = para.run(method, crd=crds, vel=vels)

    The calculated result return as generator.

    >>> for res in results:
        if para.is_root():
    ...     print(res)

    To perform test calculation, You can be run as the main module:

        $ python parallel.py

    """

    def __init__(self):
        self.__iproc = 0
        self.__body_times = []

    def is_root(self):
        return True

    def get_rank(self):
        return 0

    def get_nproc(self):
        return 1

    def _gen_kwds(self, kwds):
        """Generate each dictionary from listed dictionaries."""

        ks = kwds.keys()    # kwd1, kwd2, ...
        npair = len(ks)     # the number of kwds
        vs = kwds.values()  # val1_iter, val2_iter, ...
        pair_vs_iter = it.izip(*vs)

        for pair_vs in pair_vs_iter:
            # (val1, val2, ...)_1, (val1, val2, ...)_2, ...
            yield dict( (ks[i], pair_vs[i]) for i in range(npair) )

    def run(self, method, *args, **kwds):
        """Generate result after run in Serial."""

        method_times = []

        kwds_iter = self._gen_kwds(kwds)
        istep = None
        for istep, key_to_datas in enumerate(self._gen_kwds(kwds)):

            t1 = time.time()
            result = method(**key_to_datas)
            t2 = time.time()

            # store time intervals for each process
            method_times.append(t2-t1)

            yield result

        self.__nstep = istep if istep else 0
        self.__body_times  = method_times

    def get_time_info(self, nstep):
        t = sum(time for time in self.__body_times) / float(nstep)
        return [ ('Body' , t ) ]

    def get_other_time(self):
        return None

    def write(self, *args, **kwds):
        print(*args, **kwds)


def main():


    def method(crd, vel):
        import time
        time.sleep(0.1)
        return crd*vel

    import time
    t1 = time.time()

    crds = ( i+1 for i in range(1000))
    vels = ( 10*(i+1) for i in range(1000))
    para = ParallelProcessor()

    if para.is_root():
        t2 = time.time()
        print('elasped time : t2 - t1 : {}'.format(t2-t1))

    results = para.run(method, crd=crds, vel=vels)
    if para.is_root():
        t3 = time.time()
        print('elasped time : t3 - t2 : {}'.format(t3-t2))

    for r in results:
        # if para.is_root():
        r

    if para.is_root():
        t4 = time.time()
        print('elasped time : t4 - t3 : {}'.format(t4-t3))

def main_list(method, natom, ntraj):

    import time
    t1 = time.time()

    # initialization
    import copy
    crd  = [ (iatm+1, iatm+2, iatm+3) for iatm in range(natom) ]
    crds = ( copy.copy(crd) for itraj in range(ntraj) )
    vel  = [ (10*(iatm+1), 10*(iatm+2), 10*(iatm+3)) for iatm in range(natom) ]
    vels = ( copy.copy(crd) for itraj in range(ntraj) )

    run_bench(method, crds, vels, 'List', t1)

def main_array(method, natom, ntraj):

    import time
    t1 = time.time()

    # initialization
    import copy
    import numpy
    crd = numpy.ones([natom,3])

    crds = ( crd.copy() for itraj in range(ntraj) )
    vel = numpy.ones([natom,3])
    vels = ( vel.copy() for itraj in range(ntraj) )

    run_bench(method, crds, vels, 'Array', t1)

def main_arraygen(natom, ntraj):
    # initialization
    import copy
    import numpy

    par = ParallelProcessor()
    label = 'Array 2'

    import time
    t1 = time.time()

    def method(data):
        import time
        time.sleep(0.5)
        itraj, (crd, vel) = data
        return itraj

    if par.is_root():

        crdvel = numpy.ones([natom,3])

        def gen_data_iter(ntraj):
            for itraj in range(ntraj):
                yield itraj, (crdvel.copy(), crdvel.copy())

        data_iter = gen_data_iter(ntraj)
    else:
        data_iter = None
        # def gen_none_iter():
            # while True:
                # yield None
        # data_iter = gen_none_iter()

        # def gen_none_iter(ntraj):
            # for itraj in range(ntraj):
                # yield None

        # data_iter = gen_none_iter(ntraj)

    t2 = time.time()
    msg = 'elasped time : {} preparation     : {:8.5f}'.format(label, t2-t1)
    par.write(msg)

    gen_result = par.run(method, data=data_iter)

    t3 = time.time()
    msg = 'elasped time : {} passed into run : {:8.5f}'.format(label, t3-t2)
    par.write(msg)

    # if par.is_root():
    for r in gen_result:
        # par.write(r) 
        # if par.is_root():
            print(r)

    t4 = time.time()
    msg = 'elasped time : {} perform the run : {:8.5f}'.format(label, t4-t3)
    par.write(msg)


def main_arraygen(natom, ntraj):
    # initialization
    import copy
    import numpy

    par = ParallelProcessor()
    label = 'Array 2'

    import time
    t1 = time.time()

    def method(data):
        import time
        time.sleep(0.5)
        itraj, (crd, vel) = data
        return itraj

    if par.is_root():

        crdvel = numpy.ones([natom,3])

        def gen_data_iter(ntraj):
            for itraj in range(ntraj):
                yield itraj, (crdvel.copy(), crdvel.copy())

        data_iter = gen_data_iter(ntraj)
    else:
        data_iter = None
        # def gen_none_iter():
            # while True:
                # yield None
        # data_iter = gen_none_iter()

        # def gen_none_iter(ntraj):
            # for itraj in range(ntraj):
                # yield None

        # data_iter = gen_none_iter(ntraj)

    t2 = time.time()
    msg = 'elasped time : {} preparation     : {:8.5f}'.format(label, t2-t1)
    par.write(msg)

    gen_result = par.run(method, data=data_iter)

    t3 = time.time()
    msg = 'elasped time : {} passed into run : {:8.5f}'.format(label, t3-t2)
    par.write(msg)

    # if par.is_root():
    for r in gen_result:
        # par.write(r) 
        # if par.is_root():
            print(r)

    t4 = time.time()
    msg = 'elasped time : {} perform the run : {:8.5f}'.format(label, t4-t3)
    par.write(msg)


def run_bench(method, crds, vels, label, t):

    import time

    para = ParallelProcessor()

    t2 = time.time()
    msg = 'elasped time : {} preparation     : {:8.5f}'.format(label, t2-t)
    para.write(msg)

    gen_result = para.run(method, crd=crds, vel=vels)
    t3 = time.time()
    msg = 'elasped time : {} passed into run : {:8.5f}'.format(label, t3-t2)
    para.write(msg)

    for r in gen_result:
        # if para.is_root():
        # para.write(r)
        para.write(para.get_rank(),r)

    t4 = time.time()
    msg = 'elasped time : {} perform the run : {:8.5f}'.format(label, t4-t3)
    para.write(msg)


if __name__ == '__main__':
    natom = 10**5
    ntraj = 16

    def method(crd, vel):
        import time
        time.sleep(0.5)
        # import utility
        # return utility.tensor(crd, vel)
        return len(crd)


    main_list(method, natom, ntraj)
    main_array(method, natom, ntraj)
    main_arraygen(natom, ntraj)



from __future__ import print_function
import sys
import math
import numpy


class TensorParser:

    section_end = '%end'

    def __init__(self, filename):
        self.__filename = filename

    def parse(self):
        with open(self.__filename, 'rb') as file:
            names, tensors = self.parse_content(file)
        return names, tensors

    def parse_content(self, file):

        tensors = []

        gen_lines = self.gen_optimized_lines(file)
        sections = []
        for line in gen_lines:
            if not self.is_section(line): continue

            flagname = line[1:].strip()
            if flagname == 'data':
                lines = list( self.gen_section_lines(gen_lines) )
                names, tensor = self.parse_data(lines)
                tensors.append( tensor )

        return names, tensors

    def parse_data(self, lines):
        names = []
        tensors = []
        for line in lines:
            cols = line.split()
            names.append(cols[0])
            v = [float(c) for c in cols[1:]]
            tensor = [ [v[0], v[1], v[2]],
                       [v[3], v[4], v[5]],
                       [v[6], v[7], v[8]] ]
            tensors.append(tensor)
                
        # array = numpy.array(v)
        # print(array.reshape(3,3,len(array)))
        return names, numpy.array(tensors)

    def gen_optimized_lines(self, file):
        # first flag
        for line in file:
            if self.is_section(line):
                yield line
                break
            yield line

        for line in file:
            if self.is_section(line):
                yield self.section_end
                yield line
            yield line

        else:
            yield self.section_end

    def split_content(self, gen):
        lines = []
        for line in gen:
            if line.startswith('%'):
                pass

    def is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('%'):
            return True
        else:
            return False

    def gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            elif self.is_section(line):
                continue
            yield line


def split_data(data, interval=100):

    splitted_data = []
    num_data = len(data)
    num_intvls = num_data/interval

    for nint in range(num_intvls):
        split_data.append( data[nint*interval:(nint+1)*interval] )

    # The remains of data are thrown away.
    return splitted_data

def cal_average(tensors):

    sum_tensor = numpy.zeros(tensors[0].shape)

    for t in tensors:
        sum_tensor += t

    nten = len(tensors)
    return sum_tensor/nten

def cal_rmsf(tensors, average_tensor):

    rmsf_tensor = numpy.zeros(average_tensor.shape)

    for t in tensors:
        diff_ten = average_tensor - t
        rmsf_tensor += diff_ten * diff_ten

    nten = len(tensors)

    return numpy.sqrt(rmsf_tensor) / nten

def cal_contribution_ratio(vals):
    """Calculate the contribution ratio each component."""
    total = sum(vals)
    return [v/total * 100.0 for v in vals]

def gen_eigens(tensors):
    """Generate the calculated eigen values for each tonsor."""
    # diagonal
    for tensor in tensors:
        eigen, vec = numpy.linalg.eig(tensor)
        yield math.sqrt(eigen[0]**2 + eigen[1]**2 + eigen[2]**2)


if __name__ == '__main__':

    # make argument parser
    import argparse
    parser = argparse.ArgumentParser(
            'Show and make figure for stress ratio.')
        print('simplify_tensor total_file "label1,label2,label3,label4,..."',
                'label1_data,label2_data,...')

    # add argument definitions
    parser.add_argument(
            '-i', '--input-data', dest='data_filename', required=True,
            help='specify input filename for the stress data.')
    parser.add_argument(
            '-l', '--label-line', dest='label_line',
            required=False, default='',
            help='specify label string.')
    parser.add_argument(
            '-o', '--outputs', dest='fns',
            required=False, action='store_true', default=False,
            help='specify filenames.')

    # make arguments
    args = parser.parse_args()

    if len(parser.label_line.split(',')) != len(parser.fns.split(',')):
        print("The number of labels and filenames must be same.")
        quit()

    filename = args.data_filename
    label_line = args.label_line
    fns = args.split(',')

    # for total stress tonsor
    tot_parser = TensorParser(filename)
    names, tot_data = tot_parser.parse()
    tot_data_spl = splitted_data(tot_data)
    for tot_data in tot_data_spl:
        ave_tensors = cal_average(tot_data)
        tot_evalues = list(gen_eigens(ave_tensors))

    # for each component of stress tensor
    labels = [ label.strip() for label in label_line.split(',') ]
    
    evalues = []
    for lebel, fn in zip(labels, fns):
        parser = TensorParser(fn)
        names, data = parser.parse()
        average_tensors = cal_average(data)
        evalues.append( list(gen_eigens(average_tensors)) )

    # calculate for each interval
    for tot_data in -w
        ave_tensors = cal_average(tot_data)
        tot_evalues = list(gen_eigens(ave_tensors)


    # output the eigen value of stress tensor
    if len(evalues) == 0:
        header = '{id:9s}  {name:>4s} {total:>12s}'.format(
                id='#label id', name='name', total='total')
        print(header)
        fmt = '{id:>9d}  {name:<4s} {total:12.7f}'

        for name, tot_evalue in zip(names, tot_evalues):

            # name
            atom_id, aname = name.split('_')

            # output one line data
            print(fmt.format(id=int(atom_id), name=aname, total=tot_evalue))

    else:
        header = '{id:9s}  {name:>4s} {total:>12s}'.format(
                id='#label id', name='name', total='total') + ' '.join(
                        '{:>10s}'.format(label) for label in labels)
        print(header)
        fmt = '{id:>9d}  {name:<4s} {total:12.7f}' + ' '.join(
                '{'+label+':10.2f}' for label in labels)

        for tuples in zip(names, tot_evalues, *evalues):

            name, tot_evalue, evalue_tup = tuples[0], tuples[1], tuples[2:]

            # name
            atom_id, aname = name.split('_')

            # each evalue
            if not isinstance(evalue_tup, tuple):
                evalue_tup = [evalue_tup]
            ratios = cal_contribution_ratio(evalue_tup)
            label_to_ratios = dict(zip(labels, ratios))

            # output one line data
            print(fmt.format(id=int(atom_id), name=aname,
                total=tot_evalue, **label_to_ratios))




#! /usr/bin/env python
from __future__ import print_function

import numpy

from wurp.script.lib_distance import distance

def do_distance(tpl, trj, trj_type,
                dist_format=None, dist_cutoff=5.0, dist_method='com', **kwds
                ):

    if not trj_type == 'crd':
        msg = 'Distance method is valid only for coordinate trajectory.'
        raise Exception(msg)

    from wurp.table.group import gen_residue_group

    # Generate residue group
    res_info  = tpl.get_residue_info()
    atom_info = tpl.get_atom_info()
    rname_iatoms_pairs = list( (rname, list(numpy.array(iatoms)))
            for rname, iatoms in gen_residue_group(
                res_info, atom_info, name_fmt=dist_format))

    resnames = [ resname for resname, iatoms in rname_iatoms_pairs ]
    nres = len(resnames)

    res_beg_end_pairs = [ (min(iatoms), max(iatoms))
            for rname, iatoms in rname_iatoms_pairs ]

    mod = distance
    mod.res_beg_end_pairs = numpy.array(res_beg_end_pairs)
    mod.cutoff2 = dist_cutoff ** 2
    if dist_method == 'com':        mod.method = 1
    elif dist_method == 'nearest':  mod.method = 2
    elif dist_method == 'farthest': mod.method = 3
    else: pass

    dist2s_trj = (get_dist2s(crd, nres) for ifrm, crd, box in trj )

    # Calculate average and rms of the distances
    dists_min, dists_avg, dists_max, dists_rms = get_min_avg_max_rms(dist2s_trj)

    # Write data
    write_distance(dists_min, dists_avg, dists_max, dists_rms, resnames)

def get_dist2s(crd, nres):
    dist2s = distance.cal_dist2s(crd, nres)
    return dist2s

def get_min_avg_max_rms(dist2s_trj):

    import time
    t0 = time.time()
    print('# itraj = {:>8}'.format(1), end='')
    dist2s_fst = dist2s_trj.next()
    dist2s_sum = dist2s_fst
    dists_sum  = numpy.sqrt(dist2s_fst)

    dist2s_min = dist2s_fst.copy()
    dist2s_max = dist2s_fst.copy()

    t1 = time.time()
    print('{:12.4f}'.format(t1-t0))
    t0 = t1
    for itrj, dist2s in enumerate(dist2s_trj, 2):
        print('# itraj = {:>8}'.format(itrj), end='')

        dist2s_sum += dist2s
        dists_sum  += numpy.sqrt(dist2s)
        dist2s_min = numpy.minimum(dist2s_min, dist2s)
        dist2s_max = numpy.maximum(dist2s_max, dist2s)

        t1 = time.time()
        print('{:12.4f}'.format(t1-t0))
        t0 = t1

    nframe = itrj

    dists_avg = dists_sum/float(nframe)
    dists_rms = numpy.sqrt(dist2s_sum/float(nframe) - dists_avg**2)
    return numpy.sqrt(dist2s_min), dists_avg, numpy.sqrt(dist2s_max), dists_rms

def write_distance(dmin, davg, dmax, drms, resnames):

    nres = len(resnames)
    '# write distance data'
    header_fmt = '# {:>12} {:>12} {:>12s} {:>12s} {:>12s} {:>12s}'
    data_fmt   = '  {:>12} {:>12} {:>12.3f} {:>12.3f} {:>12.3f} {:>12.3f}'
    print(header_fmt.format(
        'residue 1', 'residue 2', 'min_dist', 'avg_dist', 'max_dist', 'rms'))
    for ires_1 in range(nres-1):
        res_i = resnames[ires_1]
        for jres_1 in range(ires_1+1, nres):
            res_j = resnames[jres_1]
            print(data_fmt.format(
                res_i, res_j, dmin[ires_1,jres_1], davg[ires_1,jres_1],
                dmax[ires_1,jres_1], drms[ires_1,jres_1]))

    print('# Finished completely.')

def load_mask(fn):

    with open(fn, 'rb') as file:
        ids_str = (line.split()[1] for line in file
                if line.startswith('ATOM'))

        ids = [ int(id_str)-1 for id_str in ids_str ]

    return ids

if __name__ == '__main__':
    pass



from __future__ import print_function
import sys
import numpy

from wurp.script.cal_tc import get_stringnames
from wurp.script.sum_acf import load_acf_first

import netCDF4 as netcdf



def gen_indices(grange_lines):
    for grange_line in grange_lines:
        fst, lst = grange_line.split(':')
        if float(int(fst))<=0:
            msg = 'The start index is invalid: {}.'.format(fst)
            raise Exception(msg)

        for index in range(int(fst), int(lst)+1):
            yield index


def write_data(prefix, donor, acceptor, acf, times):
    fn = '{:}-{:}-{:}.dat'.format(prefix, donor, acceptor)
    numpy.savetxt(fn, numpy.array([times, acf]).T, fmt='%12.4f   % 12.7e')


def get_ncdata(acf_fp, group_ranges, dataname='acf', prefix='acf', **kwds):
    """Get simple text data from file in netcdf format by given arguments."""

    index_iter = gen_indices(group_ranges)

    # first step
    times, donors, acceptors, acfs = load_acf_first(acf_fp, dataname)

    for i in index_iter:
        i_1 = i-1
        donor, acceptor, acf = donors[i_1], acceptors[i_1], acfs[i_1]
        write_data(prefix, donor, acceptor, acf, times)


if __name__ == '__main__':
    from wurp.console import arg_ncdata, exec_command

    parser = arg_ncdata()
    exec_command(parser)



"""Script console arguments."""

from __future__ import print_function

#Standard module
import argparse

import wurp


manual = {
          'conv-trj': 'Various scripts to process and analyze trajectories.',
          'analyze': 'Various scripts to analyze CURP results.',
          # Descriptions used in analyze
          'divide-flux': ('Divide the flux data of all into the flux data for '
                          'each donor and acceptor.'),

          'pickup-respairs': ('Pick up the residue pair table to given file '
                              'as a ndx format.'),

          'simplify-tensor': 'Show and make figure for stress ratio.',

          'sum-tc': 'Summarize transport coefficient',

          'get-ncdata': ('Get simple text data from file in netcdf format '
                         'by given arguments.')
          }


def top_n_trj(func):
    """Decorator to make topology and trajectory objects before the function.

    Most functions called by conv-trj command need coordinate and trajectory
    objects. This decorator include their creation before calling the function.

    Parameters
    ----------
    func: function or method
    """

    def new_func(**kwds):

        if kwds['is_crd']:
            kwds['trj_type'] = 'crd'
        elif kwds['is_vel']:
            kwds['trj_type'] = 'vel'

        # Get topology and parameter file.
        tpl = wurp.get_tpl(kwds['tpl_fmt'], kwds['tpl_fp'])
        # Get trajectory.
        trj = wurp.gen_trj(tpl, **kwds)

        func(tpl, trj, **kwds)

    return new_func


def arg_conv_trj(parser=None):
    """Parse command line arguments of wurp conv-trj"""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['conv-trj']
    parser.prog = 'conv-trj'

    # Add subparsers
    sp = parser.add_subparsers(dest='proc_name',
                               help='Processing command help')

    convert_only = sp.add_parser('convert-only',
                                 help=('Convert the trajectory into'
                                       'other format.'))
    dryrun = sp.add_parser('dry-run',
                           help='Do dry-run mode.')
    adjust_vel = sp.add_parser('adjust-vel',
                               help=('Adjust time of the velocity '
                                     'trajectory to t from t-dt/2.'))
    mask = sp.add_parser('mask',
                         help='Remove solvent from trajectory.')
    dist = sp.add_parser('dist',
                         help='Calculate inter-residue distances.')

    # Set process parcers arguments
    # parser.set_defaults(func=...) associate a function to the subparser
    # top_n_trj is a decorator to get the topology and the trajectory.

    convert_only.set_defaults(
        func=top_n_trj(wurp.script.do_convert_only))

    dryrun.set_defaults(
        func=top_n_trj(dryrun))

    adjust_vel.set_defaults(
            func=top_n_trj(wurp.script.adjust_vel))

    mask.set_defaults(
            func=top_n_trj(wurp.script.do_mask))
    mask.add_argument(
            '-m', '--mask-filename', metavar='MASK_FILENAME',
            dest='mask_fn', required=True,
            help=('The mask file name.'))

    dist.set_defaults(
            func=top_n_trj(wurp.script.do_distance))
    dist.add_argument('-m', '--method', metavar='DISTANCE_METHOD',
                      dest='dist_method', required=True,
                      choices=['cog', 'nearest', 'farthest'],
                      help=('The method used in calculating '
                            'the inter-residue distances.')
                      )
    dist.add_argument('-c', '--cutoff-length', metavar='CUTOFF_LENGTH',
                      dest='dist_cutoff', required=True,
                      default='5.0', type=float,
                      help=('The distance cutoff in angstrom.'))

    # Add common arguments
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
            '-crd',  # metavar='TRJ_TYPE',
            dest='is_crd', required=False, action='store_true',
            help=("Specify the format of the trajectory."
                  "This argument allows you specify multiple formats."))

    group.add_argument(
            '-vel',  # metavar='TRJ_TYPE',
            dest='is_vel', required=False, action='store_true',
            help=("Specify the format of the trajectory."
                  "This argument allows you specify multiple formats."))

    # add argument definitions
    parser.add_argument(
            '-i', '--input-filenames', metavar='TRJ_FILENAME',
            dest='input_trj_fns', required=True, nargs='+', action='append',
            help=('The trajectory file names.'))

    parser.add_argument(
            '-if', '--input-formats', metavar='FORMAT',
            dest='input_trj_fmts', required=True, action='append',
            help=("Specify the format of the trajectory."
                  "This argument allows you specify multiple formats."))

    parser.add_argument(
            '--irange', metavar=('FIRST', 'LAST', 'INTER'),
            dest='input_fst_lst_int', required=False,
            default=None, type=int, nargs=3,  action='append',
            help='The trajectory range to process over all trajectory file.')

    parser.add_argument('-o', '--output-filename', metavar='TRJ_FILENAME',
                        dest='output_trj_fn', required=False,
                        help=('The trajectory file name.'))

    parser.add_argument('-of', '--output-format', metavar='FORMAT',
                        dest='output_trj_fmt', required=False,
                        help=('The trajectory file format for output.'))

    parser.add_argument('--orange', metavar=('FIRST', 'LAST', 'INTER'),
                        dest='output_fst_lst_int', required=False,
                        default=[0, -1, 1], type=int, nargs=3,
                        help=('The trajectory range for output'))

    parser.add_argument('-pf', '--topology-format', metavar='FORMAT',
                        dest='tpl_fmt', required=True,
                        help='Specify the format of the topology file.')

    parser.add_argument('-p', '--topology-file', metavar='TPL_FILENAME',
                        dest='tpl_fp', required=True,
                        help='Specify the topology file.')

    parser.add_argument(
            '-nf', '--name-format', dest='dist_format',
            required=False, default='{rid:05}_{rname}',
            help='Specify the format for representing residue identify.')

    if return_parser:
        return parser


def arg_analyze(parser=None):
    """Parse command line arguments of wurp analyze"""

    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['analyze']
    parser.prog = 'analyze'

    # Definitions
    sp = parser.add_subparsers(dest='process',
                               help=manual['analyze'])
    sp_divide = sp.add_parser('divide-flux',
                              help=manual['divide-flux'])
    sp_respairs = sp.add_parser('pickup-respairs',
                                help=manual['pickup-respairs'])
    sp_simplify = sp.add_parser('simplify-tensor',
                                help=manual['simplify-tensor'])
    sp_sum_tc = sp.add_parser('sum-tc',
                              help=manual['sum-tc'])
    sp_ncdata = sp.add_parser('get-ncdata',
                              help=manual['get-ncdata'])

    arg_divide_flux(sp_divide)
    arg_respairs(sp_respairs)
    arg_simplify(sp_simplify)
    arg_sum_tc(sp_sum_tc)
    arg_ncdata(sp_ncdata)

    if return_parser:
        return parser


# wurp analyze subparsers
def arg_divide_flux(parser=None):
    """Parse command line arguments of wurp analyze divide-flux"""
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['divide-flux']
    parser.proc = 'divide-flux'
    parser.set_defaults(func=wurp.script.analyze.divide_flux)

    # Argument definition for divide
    parser.add_argument(
            'flux_fns', nargs='+',
            help=('Specify filenames in order. '
                  'ex.) flux.dat0001, flux.dat0002, ...'))
    parser.add_argument(
            '-o', '--output-filename', dest='output_fn', required=True,
            help='Specify a filename to output.')
    parser.add_argument(
            '-d', '--donor', dest='donor_line', required=False, default='',
            help='Donor names to output to files.')
    parser.add_argument(
            '-a', '--acceptor', dest='acceptor_line', required=False,
            default='',
            help='Acceptor names to output to files.')
    parser.add_argument(
            '-t', '--dt', dest='dt', required=True, default=1.0,
            help='Time width[ps] per 1 step.')
    parser.add_argument(
            '-c', '--column', dest='column_line', required=False, default='',
            help='Column numbers to output.')

    if return_parser:
        return parser


def arg_respairs(parser=None):
    """Parse command line arguments of wurp analyze pickup-respairs"""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.definition = manual['pickup-respairs']
    parser.proc = 'pickup-respairs'
    parser.set_defaults(func=wurp.script.analyze.pickup_respairs)

    parser.add_argument(
            dest='trj_fns', nargs='+', action='append',
            help=('The trajectory file names.'))
    parser.add_argument('-if', '--input-format', metavar='FORMAT',
                        dest='input_trj_fmt', required=True, action='append',
                        help='Specify formats of trajectories.')
    parser.add_argument(
            '-p', '--input-prmtop-file', dest='prmtop_fn', required=True,
            help='Specify the prmtop file to be amber format.')
    parser.add_argument(
            '-pf', '--input-prmtop-format', dest='prmtop_fmt', required=True,
            help='Specify the format of the prmtop file.')
    parser.add_argument(
            '-i', '--interval', dest='interval', default=1, type=int,
            help=('Specify interval step to perform the calculation for '
                  'trajectory.'))
    parser.add_argument(
            '-m', '--cutoff-method', dest='cutoff_method', default='nearest',
            required=False, choices=['com', 'nearest', 'farthest'],
            help='Cutoff method; com, nearest and farthest for residue.')
    parser.add_argument(
            '-c', '--cutoff', dest='cutoff',
            required=False, default=5.0,
            help='Specify the cutoff to pick up.')
    parser.add_argument(
            '-t', '--trim-resnames', dest='trim_resnames', nargs='*',
            required=False, default=[],
            help='Residue names for trimming from the target residues.')
    parser.add_argument(
            '-f', '--format', dest='format',
            required=False, default='{rid:05}_{rname}',
            help='Specify the format for representing residue identify.')
    parser.add_argument(
            '-U', '--disble-union', dest='is_union',
            required=False, action='store_false',
            help=('Whether residue pair table is calculated by union '
                  'or intersection when collecting over trajectoryspecify'))
    parser.add_argument('-e', '--exclude-resids', metavar='FIRST:LAST',
                        dest='ext_resids', required=False,
                        default='', nargs='*',
                        help=('Residues that you want to exclude.'
                              'ex) 5:10 80:150'))
    parser.add_argument(
            '-b', '--enable-residues-both-cxcluded', dest='is_both',
            required=False, action='store_true',
            help=('Whether include residue pairs that exclude residues'))

    if return_parser:
        return parser


def arg_simplify(parser=None):
    """Parse command line arguments of wurp analyze simplify-tensor"""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.definition = manual['simplify-tensor']
    parser.proc = 'simplify-tensor'
    parser.set_defaults(func=wurp.script.analyze.simplify_tensor)

    # add argument definitions
    parser.add_argument(
            '-i', '--input-data', dest='filename', required=True,
            help='Specify input filename for the stress data.')
    parser.add_argument(
            '-l', '--labels', dest='labels',
            required=False, default='',
            help=('Specify labels of components to analyze.'
                  'ex.) -l "total,bond,angle,..."'))
    parser.add_argument(
            'fns', nargs='*',    # action='append',
            help=('Specify additive filenames. '
                  'ex.) label_data1, label_data2, ...'))
    parser.add_argument(
            '-s', '--every-snapshot', dest='snapshot',
            required=False, action='store_true', default=False,
            help='Specify flag to average the magnitude for every snapshot.')

    if return_parser:
        return parser


def arg_sum_tc(parser=None):
    """Parse command line arguments of wurp analyze sum-tc"""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.definition = manual['sum-tc']
    parser.proc = 'sum-tc'
    parser.set_defaults(func=wurp.script.analyze.summarize_tc)

    # add argument definitions
    parser.add_argument('tc_fps', nargs='+',
                        help='Data files')

    if return_parser:
        return parser


def arg_ncdata(parser=None):
    """Parse command line arguments of wurp analyze get-ncdata"""

    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.definition = manual['get-ncdata']
    parser.proc = 'get-ncdata'
    parser.set_defaults(func=wurp.script.analyze.get_ncdata)

    parser.add_argument(dest='acf_fp', metavar='ACF_FILE',
                        help='The filepath of auto-correlation function data.')
    parser.add_argument('-r', '--group-ranges', metavar='FRIST:LAST',
                        nargs='*',
                        dest='group_ranges', required=True,
                        default='',
                        help='The pair range list to want to gain.')
    parser.add_argument('-n', '--dataname',
                        dest='dataname', required=False,
                        default='acf',
                        help='The name of the netcdf data you want to gain.')
    parser.add_argument('-o', '--output-prefix',
                        dest='prefix', required=False,
                        default='acf',
                        help=('The prefix of the files to want to write, '
                              'that includes directory path.'))

    if return_parser:
        return parser





#! /usr/bin/env python2
from __future__ import print_function

import os, sys

def parse_asa(filename):
    """
    1
    2
    3
    5
    9
    .
    .
    341
    342
    """

    file = open(filename, 'rb')

    for line in file:
        if line.startswith('#'): continue
        yield int(line.strip())

    file.close()


def main(ec_fn, asa_fn, **kwds):
    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
        lines = (line for line in ec_file
                if not line.startswith('#')
                if not line.isspace() )
    ec_file.close()

    for line in lines:
        # if line.startswith('#'):
            # print(line.strip())
            # continue
        # print(line)

        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        rid1 = int(don_line.split('_')[0])
        rid2 = int(acc_line.split('_')[0])

        if rid1 in asa_resids or rid2 in asa_resids:
        # if rid1 in asa_resids and rid2 in asa_resids:
            # print(rid1, rid2)
            continue

        else:
            print(line.strip())

if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_noasa
    parser = arg_sel_noasa()
    exec_command(parser)




import trajectory
coordinate_dict = {'presto-bin' : trajectory.CoordinateParser}
velocity_dict   = {'presto-bin' : trajectory.VelocityParser}
restart_dict    = {'presto-bin' : trajectory.RestartParser}

import topology
topology_dict  = {'presto'  : topology.TopologyParser}
converter_dict = {('presto', 'amber99') : topology.Format2Amber99Converter}




from pymol import cmd
import sys, os

ONE_LETTER = {
        'VAL':'V', 'ILE':'I', 'LEU':'L', 'GLU':'E', 'GLN':'Q',
        'ASP':'D', 'ASN':'N', 'HIS':'H', 'TRP':'W', 'PHE':'F', 'TYR':'Y',
        'ARG':'R', 'LYS':'K', 'SER':'S', 'THR':'T', 'MET':'M', 'ALA':'A',
        'GLY':'G', 'PRO':'P', 'CYS':'C',
        'HIE':'H', 'HID':'H', 'HIP':'H', 'ASH':'D', 'GLH':'E',
        'WAT':'w', 'HOH':'w',
        }

def main():

    args = get_args()

    ecs = list(parse_ec(args.ec_fps[0]))

    inc_ecs, dec_ecs = [], []
    for data in ecs:
        ec = float(data['values'][0])
        if ec >= args.threshold:
            inc_ecs += [(data, ec)]
        elif ec <= - args.threshold:
            dec_ecs += [(data, ec)]
        else:
            pass
    
    if args.use_increase:
        for d, ec in inc_ecs:
            display_pair(d, ec, args)

    if args.use_decrease:
        for d, ec in dec_ecs:
            display_pair(d, ec, args)

    if args.output_fp:
        cmd.ray(args.resolution)
        cmd.png(args.output_fp)
        cmd.quit()

    if args.movie_style:

        # set resolution
        x, y = cmd.get_viewport(output=1)
        cmd.viewport(args.resolution, args.resolution*y/x)

        # perform ray trace in write the movie
        cmd.set("ray_trace_frames", 1)

        if args.movie_style == 'y-rotation':
            cmd.mset(1, 120)
            cmd.movie.roll(1,120,1,axis='y')
            cmd.mpng("_movie/")
            cmd.quit()

        elif args.movie_style == 'x-rotation':
            cmd.mset(1, 120)
            cmd.movie.roll(1,120,1,axis='x')
            cmd.mpng("_movie/")
            cmd.quit()

        elif args.movie_style == 'y-rock':
            cmd.mset(1, 120)
            cmd.movie.rock(1, 120, 40, axis='y')
            cmd.mpng("_movie/")
            cmd.quit()

        elif args.movie_style == 'x-rock':
            cmd.mset(1, 120)
            cmd.movie.rock(1, 120, 40, axis='x')
            cmd.mpng("_movie/")
            cmd.quit()

        elif args.movie_style == 'nutate':
            cmd.mset(1, 120)
            cmd.movie.nutate(1, 120, 40)
            cmd.mpng("_movie/")
            cmd.quit()

        else:
            pass


def get_args():
    """Parse and get the command line arguments."""
    import argparse
    parser = argparse.ArgumentParser(
            description="Show 3D structure with EEN.")

    parser.add_argument(
            dest='ec_fps', nargs='+', metavar='EC_FP',
            help="specify the ec file paths."
            )

    # parser.add_argument(
            # '-r', "--important-residues", dest='residues',
            # metavar=('res1','res2'), default=[], type=int, nargs='+',
            # help="specify residues that you want to highlight.",
            # )

    parser.add_argument(
            "-L", "--without-residue-labels", dest="enable_labels",
            action="store_false",
            help="enable the labels of residues."
            )

    parser.add_argument(
            "-I", "--not-dispaly-increase", dest="use_increase",
            action="store_false",
            help="don't display increasing pairs."
            )

    parser.add_argument(
            "-d", "--dispaly-decrease", dest="use_decrease",
            action="store_true",
            help="display dncreasing pairs."
            )

    parser.add_argument(
            "-t", "--threshold", dest="threshold",
            default=0.003, type=float,
            help="specify the threshold value that show on 3D structure.",
            )

    parser.add_argument(
            "-n", "--node-color", dest="node_color",
            default='grey', type=str,
            help="specify the color of the residues."
            )

    parser.add_argument(
            "-o", "--output-filepath", dest="output_fp",
            default="",
            help="specify a file path to write figure."
            )

    parser.add_argument(
            "-r", "--resolution", dest="resolution",
            default=820, type=int,
            help="specify the resolution of a file path to write figure."
            )

    # parser.add_argument(
            # '-m', "--movie-filename", dest="movie_fp",
            # default="",
            # help="specify a file path to write a movie."
            # )

    parser.add_argument(
            '-ms', "--movie-style", dest="movie_style",
            default="y-rotaiton",
            choices=["y-rotation", "x-rotation", "x-rock", "y-rock", "nutate"],
            help="specify the style of the movie."
            )

    args = parser.parse_args()
    return args

def parse_ec(ec_fp):

    file = open(ec_fp, 'r')

    lines = (line.strip() for line in file
            if line
            if not line.startswith('#') )

    for line in lines:

        cols = line.split()
        res1, res2, values = cols[0], cols[1], cols[2:]

        rid1, rname1 = res1.split('_')
        rid2, rname2 = res2.split('_')

        yield dict(
                rid1=int(rid1), rname1=rname1,
                rid2=int(rid2), rname2=rname2,
                values = values )

    file.close()

def highlight_important_residues(filename, residue_shift=0):

    colors = ['red', 'orange', 'yellow', 'green']

    file = open(filename, 'b')

    # remove the only space line and the comment line
    line_iter = ( line.strip() for line in file
            if not line.isspace() if not line.startswith('#') )

    for line in line_iter:
        cols = line.split()
        rid, irank = int(cols[0]), int(cols[1])
        sel = str(rid-residue_shift) + '/CA'

        # show spheres
        cmd.show('spheres', sel)
        cmd.label(sel)

        # coloring to spheres
        cmd.set('sphere_color', colors[irank-1], sel)
        cmd.set('sphere_scale', 0.6, sel)

    file.close()


def get_selection(rid, rname):

    if rname == 'WAT':
        sel = '(resi ' + str(rid) + ' and name O)'
    elif rname == 'CAU':
        sel = '(resn CAU and name O2)'
    else:
        sel = str(rid) + '/CA'

    return sel

def display_pair(data, ec, args):

    is_inc = ec > 0
    ec = abs(ec)

    sel1 = get_selection(data['rid1'], data['rname1'])
    sel2 = get_selection(data['rid2'], data['rname2'])

    # connect pair by ec value
    rn1 = ONE_LETTER.get(data['rname1'])
    rn2 = ONE_LETTER.get(data['rname2'])
    rid1 = str(data['rid1'])
    rid2 = str(data['rid2'])
    name1 = rn1+rid1 if rn1 else rid1
    name2 = rn2+rid2 if rn2 else rid2
    name  = "{}-{}".format(name1, name2)

    cmd.distance(name, selection1=sel1, selection2=sel2)
    cmd.hide("label", name)

    if 0.003 <= ec < 0.008:
        cmd.set('dash_width', 4, name)
        # cmd.set('dash_color', 'orange', name)
        cmd.set('dash_color', 'green', name)
    elif 0.008 <= ec < 0.015:
        cmd.set('dash_width', 4, name)
        cmd.set('dash_color', 'blue', name)
        # cmd.set('dash_color', 'white', name)
    elif 0.015 <= ec:
        cmd.set('dash_width', 6, name)
        cmd.set('dash_color', 'red', name)
    else:
        pass

    cmd.set('dash_gap', 0.8, name)
    if args.use_increase and args.use_decrease:
        if is_inc: cmd.set('dash_gap', 0.0, name)

    # show residues consisting of the pair 
    # cmd.show('sticks', sel1.replace('/CA','/'))
    # cmd.show('sticks', sel2.replace('/CA','/'))
    # cmd.color('blue', sel1.replace('/CA','/'))

    cmd.show('spheres', sel1)
    cmd.show('spheres', sel2)
    cmd.set('sphere_scale', 0.4, sel1)
    cmd.set('sphere_scale', 0.4, sel2)
    cmd.color(args.node_color, sel1)
    cmd.color(args.node_color, sel2)

    # label the pair
    if args.enable_labels:
        cmd.label(sel1, '"{}"'.format(name1))
        cmd.label(sel2, '"{}"'.format(name2))
        cmd.set("label_size", 20)
        # cmd.set("label_position", (-1.0,-1.0,-1.0))

    else:
        cmd.hide('labels')

if __name__ == '__main__':
    main()



"""setting - Defines the Setting class

Setting class contains all sections and variables from .cfg
configuration file.
Defines all the .cfg sections as class variables (Section object)
"""

from setting_base import *


class Setting(SettingBase):
    """Settings for wurp computations

    Parameters
    ----------
    config : configparser.SafeConfigParser
    check : Bool
        Whether entries are checked or not.

    Examples
    --------
    To create a default Setting object:
    >>> import wurp.setting as st
    >>> config = st.parse_config("")
    >>> setting = st.Setting(config)

    To create a Setting object from a configuration file:
    >>> import wurp.setting as st
    >>> config = st.parse_config(path_to_cfg)
    >>> setting = st.Setting(config)

    To see the sections:
    >>> dir(setting)

    To print a usable .cfg file
    >>> print(setting)

    To print a usable .rst summary
    >>> print(format(setting, "rst"))

    setting.session_name to access only a section
    Ex: input section
    >>> setting.input

    To show only the values of the input section
    >>> print(setting.input)

    To modify a variable
    Ex: To change wurp method to energy-flux
    >>> setting.wurp.method = "energy-flux"
    """

    input = Section(

        description = "",

        format = Choice(default='amber', require=True,
            desc='The format of the various files generated by other grograms.',
            values=['presto', 'amber'], value_type=String),

        first_last_interval = List(default=(0, -1, 1), require=False,
             value_type=Int,
             desc=('The first, last and interval step to read coordinates and '
                 'velocities trajectory.'),
             ),

        use_simtime = Bool(default=True, require=False,
            desc='use the simulation time information in trajectory.'),

        group_file = File(default='group.cfg',require=False, allow_glob=False,
            desc=('The group file path to define group.'
                'if wurp.group_method == file then this definition is used.')
        ),

        # group_excluded = List(default=[], require=False, value_type=String,
        #     desc='The excluded group list specified in section name'),

    )

    input_amber = Section(

        target = Choice(default='trajectory', require=False,
            desc='The target input file to be used.',
            values=['trajectory', 'restart'], value_type=String),

        topology_file = File(default='', require=True, allow_glob=False,
            desc='Topology file path.'),

        coordinate_file = File(default='', require=False, allow_glob=True,
            desc='Coordinate trajectory file path.'),

        velocity_file = File(default='', require=False, allow_glob=True,
            desc = 'Velocity trajectory file path.'),

        restart_file = File(default='', require=False, allow_glob=False,
            desc = 'Restart file path.'),

        coordinate_format = Choice(default='ascii', require=False,
            desc='The format of coordinate file.',
            values=['ascii', 'netcdf'],
            value_type=String),

        velocity_format = Choice(default='ascii', require=False,
            desc='The format of velocity file.',
            values=['ascii', 'netcdf'],
            value_type=String),

        restart_format = Choice(default='restart', require=False,
            desc='The format of restart file.',
            values=['restart'],
            value_type=String),

        dump_parameters = Bool(default=False, require=False,
            desc='Dump the parsed Amber force field parameter set.'),

    )

    # input_presto = Section(

        # target = Choice(default='trajectory', require=False,
            # desc='The target input file to be used.',
            # values=['trajectory', 'restart'], value_type=String),

        # topology_file = File(default='', require=True, allow_glob=False,
            # desc='Topology file name.'),

        # coordinate_file = File(default='', require=False, allow_glob=True,
            # desc='Coordinate trajectory file name.'),

        # velocity_file = File(default='', require=False, allow_glob=True,
            # desc = 'Velocity trajectory file name.'),

        # restart_file = File(default='', require=False, allow_glob=False,
            # desc = 'Restart trajectory file name.'),

        # coordinate_format = Choice(default='ascii', require=False,
            # desc='The format of coordinate file.',
            # values=['ascii', 'bin'],
            # value_type=String),

        # velocity_format = Choice(default='ascii', require=False,
            # desc='The format of velocity file.',
            # values=['ascii', 'bin'],
            # value_type=String),

        # restart_format = Choice(default='ascii', require=True,
            # desc='The format of restart file.',
            # values=['ascii', 'bin'],
            # value_type=String),

    # )

    volume = Section(

        method = Choice(default='voronoi', require=False,
            desc = 'Algorithm to calculate the atomic volumes.',
            values = ['none', 'vdw', 'voronoi', 'outer'], #, 'smve'],
            value_type=String),

        # smve_rmax = Float(default=3.0, require=False,
        #     desc = ('The cutoff length for the calculation of radial '
        #     'distribution function.')),

        # smve_dr = Float(default=0.01, require=False,
        #     desc = 'The width of bin for radial distribution function.')

        # smve_interval = Int(default=1, require=False,
        #     desc = ('The step interval in reading the trajectory for radial '
        #     'distribution function.')),

        # smve_increment = Int(default=5, require=False,
        #     desc = 'Search the 1st wells every "increment" step'),

        atomic_trajectory_file = File(default='', require=False,
            allow_glob=False,
            desc='Atomic volumes trajectory file path for outer method.'),

        group_trajectory_file = File(default='', require=False,
            allow_glob=False,
            desc='Group volumes trajectory file path for outer method.'),

        voronoi_cutoff = Float(default=6.0, require=False,
            desc = ('The cutoff length that the voronoi calculation finds out'
                    'neighbour candidate particles.') ),

        voronoi_no_hydrogen = Bool(default=False, require=False,
            desc=('Flag to determine whether include hydrogen atoms'
                  'for the voronoi calculation.') ),

        voronoi_solvation = Choice(default='none', require=False,
            desc=('The kind of solvation system to sink the target system '
                  'in vacuum for the voronoi method. '
                  'The density value of the water under NPT ensemble is '
                  '0.99651 [g/cm^3] at 27 [Kelvin]'),
            values=['none', 'RANDOM20', ], #'NPT20', 'NPT25' 'NPT30'],
            value_type=String),

        voronoi_probe_length = Float(default=2.4, require=False,
            desc = ('The probe length of the solvation for the voronoi method.'
                'The water molecules within the probe length '
                'from the system are removed.')
            ),

        voronoi_output_solvation_file = File(default='', require=False,
            allow_glob=False,
            desc=('The file path to write out the solvation pdb data in the '
                  'case of voronoi_solvation == "none".'
                  'If the file path is not given, writing out is not performed.')            ),

        output_volume_file = File(default='', require=False,
            allow_glob=False,
            desc=('The file path to write out the atomic volumes trajectory. '
                  'If this value is not given, writing out is not performed.'
                  'The file written by this option can be used in the options,'
                  'atomic_trajectory_file.'),
            ),

        output_gvolume_file = File(default='', require=False,
            allow_glob=False,
            desc=('The file path to write out the group volumes trajectory. '
                  'If this value is not given, writing out is not performed.'
                  'The file written by this option can be used in the options'
                  'group_trajectory_file.'),
            ),

    )

    wurp = Section(

        potential = Choice(default='amberbase', require=True,
            desc='The potential function to calculate the pairwise forces.',
            values=['amberbase', 'amber94', 'amber96', 'amber99', 'amber99SB',
                'amber03', 'amber12SB'], # 'amber-polar'],
            value_type=String),

        target_atoms = List(default=['1-'], require=False, value_type=String,
            desc='The atom list calculated.'),

        method = Choice(default='momentum-current', require=True,
            value_type=String,
            desc=('The method of calculation.'
                '"momentum-current" calculates the stress tensor for systems. '
                '"energy-flux" calculates the energy flow for systems. '
                '"dynamics is mainly used to verify the validity of '
                ' the CURP program numerically, '
                'so its implementation is very simple'),
            values=['energy-flux', 'momentum-current',
                    'microcanonical', 'heat-flux']),
                    #'energy-current', 'stress-flux']),

        group_method = Choice(default='none', require=False,
            value_type=String,
            desc=('The method to construct the group.'
                '"united" means that hydrogen atoms are included in hevy atoms '
                'covalent to them.'
                '"residue" means that the groups are calculated by residue '
                'level. If "file" is specified, the groups definition is given'
                'by the group file in input section.'),
            values=['united', 'residue', 'file', 'none'] ),

        flux_grain = Choice(default='group', require=False,
            value_type=String,
            desc=('The grain to calculate the flux.'
                '"atom", "group" and "both" values mean that the flux '
                'for atom parirs, group pairs and both of them will be '
                'calculated, respectively.'),
            values=['atom', 'group', 'both'] ),

        decomp_group_current = Bool(default=False, require=False,
            desc=('Flag whether decompose the group current into inside and '
                  'outside contributions of group region.'
                  'This option is used for calculating momentum current') ),

        group_pair_file = File(default='',require=False, allow_glob=False,
            desc=("Path to file to define group pair. "
                  "If you didn't given, all of pairs within the targets "
                  "will be calculated.")),

        coulomb_method = Choice(default='cutoff', require=False,
            desc='The method to calculate coulomb interaction.',
            values=['cutoff',], # 'multipole'],
            value_type=String),

        coulomb_cutoff_method = Choice(default='atom', require=False,
            desc='The method to cut off the coulomb interaction.',
            values=['atom'], # 'residue-center','residue-while'],
            value_type=String),

        coulomb_cutoff_length = Float(default=99.9, require=False,
            desc='The cutoff length for the coulomb interaction.'),

        # vdw_method = Choice(default='cutoff', require=False,
        #     desc='The method to calculate van der Waals interaction.',
        #     values=['cutoff', 'multipole'], value_type=String)

        vdw_cutoff_method = Choice(default='atom', require=False,
            desc='The method to cut off the van der Waals interaction.',
            values=['atom'], # 'residue-center','residue-while'],
            value_type=String),

        vdw_cutoff_length = Float(default=99.9, require=False,
            desc='The cutoff length for the van der Waals interaction.'),

        # postproccessing = Choice(default='output', require=False,
        #     desc='The define what the post-proccessing part use.',
        #     values=['output'], # 'residue-center','residue-while'],
        #     value_type=String),

        enable_inverse_pair = Bool(default=False, require=False,
                desc=('Calculate and write out inverse pairs j <- i for flux '
                    'adding normal group pairs: i <- j. '
                    'This option is used in the case '
                    'for calculating energy flux.')),

        remove_trans = Bool(default=True, require=False,
            desc=('Remove the coordinate and velocity of translation '
                  'for the target atoms')),

        remove_rotate = Bool(default=True, require=False,
            desc=('Remove the coordinate and velocity of rotation '
                  'for the target atoms')),

        log_frequency = Int(default=10, require=False,
            desc='Log informations will be written out every given steps.'),

        # restraint
    )

    dynamics = Section(

        integrator = Choice(default='vverlet', require=True,
            desc='The integrator to want to use with the dynamics.',
            values=['vverlet', 'leapfrog'],
            value_type=String),

        dt = Float(default=0.001, require=True,
            desc='Time step to advance snapshots to next step, in ps unit.'),

        num_steps = Int(default=1, require=True,
            desc='The number of integration steps.'),

        crds_file = File(default='', require=False, allow_glob=False,
            desc=("The file path to write out the coordinates trajectory. "
                  "If empty, then don't write.")),

        vels_file = File(default='', require=False, allow_glob=False,
            desc=("The file path to write out the velocities trajectory. "
                  "If empty, then don't write.")),

        crds_frequency = Int(default=1, require=False,
            desc='The interval step to write coordinate trajectory.'),

        vels_frequency = Int(default=1, require=False,
            desc='The interval step to write velocity trajectory.'),

        trj_format = Choice(default='ascii', require=False,
            desc='The format of coordinates and velocities trajectory file.',
            values=['ascii', 'netcdf'],
            value_type=String),
        )

    output = Section(

        # current or flux data
        filename = File(default='current.dat', require=False, allow_glob=False,
            desc='The file name to output the current or flux information.'),

        format = Choice(default='ascii', require=False,
            desc='The format of flux data.',
            values=['ascii', 'netcdf'],
            value_type=String),

        decomp = Bool(default=False, require=False,
            desc=('Flag whether decompose the total current or flux '
                  'to bonded, coulomb, and van der Waals interaction.') ),

        frequency = Int(default=0, require=False,
            desc=('The frequency to create new file to write '
                'the additional file.')),

        compress = Bool(default=False, require=False,
            desc=('Flag whether compress with gnu zip, '
                'then the extension of the file name became ".gz".') ),

        # basic data
        output_energy = Bool(default=False, require=False,
            desc='Flag whether output the energy information or not.'),

        energy_file = File(default='energy.dat', require=False,
            desc='The file path to output the energy information.'),

        energy_decomp = Bool(default=False, require=False,
            desc=('Flag whether decompose the total energy '
                'to bonded, coulomb, and van der Waals interaction.') ),

        energy_freqency = Int(default=0, require=False,
            desc=('The frequency to write the energy information.')),

        energy_compress = Bool(default=True, require=False,
            desc=('Flag whether compress with gnu zip for energy_file, '
                'then the extension of the file name became ".gz".')),

        # output_force = Bool(default=False, require=False,
            # desc='Flag whether output the force information or not.'),

        # force_file = File(default='force.dat', require=False,
            # desc='The file name to output the force information.'),

        # force_decomp = Bool(default=False, require=False,
            # desc=('Flag whether decompose the total force'
                # 'to bonded, coulomb, and van der Waals interaction.') ),

        # force_freqency = Int(default=0, require=False,
            # desc='The frequency to create new file to write other file.'),

        # force_compress = Bool(default=True, require=False,
            # desc=('Flag whether compress by gnu zip, '
                # 'then the extension of the file name became ".gz".') ),

        # output_tbforce = Bool(default=False, require=False,
            # desc=('Flag whether output the tbforce information or not.'
                # 'note that the file size became too large.')),

        # tbforce_file = File(default='tbforce.dat', require=False,
            # desc='The file name to output the tbforce information.'),

        # tbforce_decomp = Bool(default=False, require=False,
            # desc=('Flag whether decompose the total tbforce'
                # 'to bonded, coulomb, and van der Waals interaction.') ),

        # tbforce_freqency = Int(default=0, require=False,
            # desc='The frequency to create new file to write other file.'),

        # tbforce_compress = Bool(default=True, require=False,
            # desc=('Flag whether compress by gnu zip, '
                # 'then the extension of the file name became ".gz".') )
    )


def parse_config(config_filename=None):
    try:
        import configparser as cp
    except ImportError:
        import ConfigParser as cp
    except:
        raise

    import cStringIO as sio
    outfile = sio.StringIO()

    if config_filename:
        with open(config_filename, 'rb') as config_file:
            for line in config_file:
                body_ = line.split('#')[0]
                body  = body_.split(';')[0]
                outfile.write(body)

        outfile.seek(0)

    config = cp.SafeConfigParser()
    config.readfp(outfile)
    outfile.close()

    # config = cp.SafeConfigParser()
    # config.read(config_filename)

    return config

if __name__ == '__main__':
    config = parse_config('run.cfg')
    config = parse_config()
    setting = Setting(config, check=False)
    # setting = Setting(config)
    # print(setting)

    print(format(setting, "rst"))



"""Write the results of wurp calculator

CURP 1.1: Ishikura, 2016. Most of the Layout  
CURP 1.2: Laprevote, 2019. Heat Flux gestion (and commenting code).

In main() of wurp.py, a writer is initialized along with the calculator.
The writer varies depending on setting options (in .cfg) contained in
a setting object.
The writer will vary because of (in this order):
 * Decided in get_writer:
  - The calculation method (momentum-current, energy or heat-flux)
 * Decided in MultiFluxWriter and MultiCurrentWriter:
  - The output file format (ascii or netcdf)
  - The use of groups or not
 * Decided in CurrentWriter, FluxWriter and NetCDFFluxWriter:
  - The compression of the output file
  - In FluxWriter and NetCDFFluxWriter, the presence of axes
    (as heat flux is a vector and energy flux a single value)

Each of the XxxWriter will format lines using the output of the
calculator for one frame, and use either Writer or AxisWriter object
to write them in a file.
"""

from __future__ import print_function

import os
import numpy


def get_writer(setting, decomp_list, target_anames, group_names,
               gpair_table=None):
    """Returns Writer object depending on the setting.wurp.method."""

    method = setting.wurp.method
    if method == 'momentum-current':
        obj = MultiCurrentWriter(
                setting=setting, decomp_list=decomp_list,
                target_anames=target_anames, group_names=group_names)

    elif method == 'energy-flux':
        obj = MultiFluxWriter(
                setting=setting, decomp_list=decomp_list,
                target_anames=target_anames, group_names=group_names,
                gpair_table=gpair_table)

    elif method == 'heat-flux':
        obj = MultiFluxWriter(
                setting=setting, decomp_list=decomp_list,
                target_anames=target_anames, group_names=group_names,
                gpair_table=gpair_table, axes=('i','j','k'))

    else:
        pass

    return obj


def write(setting, results, formats):
    key_to_writer = {}
    if setting.output.energy:
        key_to_writer['energy'] = Writer(
                energy.filename, formats['energy'],
                energy.frequency, energy.compress)

    if setting.output.force:
        key_to_writer['force'] = Writer(
                force.filename, formats['force'],
                force.frequency, force.compress)

    if setting.output.twobody_force:
        key_to_writer['twobody_force'] = Writer(
                twobody_force.filename, format['twobody_force'],
                twobody_force.frequency,twobody_forceforce.compress)

    for res in results:
        for key, writer in key_to_writer.items():
            data = res[key]
            writer.write(data)


class Writer:
    """Called to write files.

    Used in CurrentWriter.
    Parent of AxisWriter.

    Keyword arguments:
    filename -- string, will precede the name of written files
    frequency -- integer 
    compresslevel -- integer, ==0 when compress=no in .cfg file
    use_axes -- boolean, True when Writer is used in AxisWriter.
    """

    def __init__(self, filename, frequency,
                 compresslevel=6, use_axes=False):
        self._base_fn = filename
        self._nfreq = frequency
        self._ifreq = 0     # The number of the frame
        self._counter = 0

        # Choose the open function used
        self._use_zip = compresslevel != 0
        if self._use_zip:
            import gzip
            def gzopen(filename, mode):
                import gzip 
                return gzip.open(filename, mode, compresslevel)
            self._open = gzopen
        else:
            self._open = open

        self.make_filename(self._ifreq, use_axes)

    def open(self):
        return self._open(self._filename, 'ab')

    def write_header(self, headers):
        """headers -- string generator"""
        file = self.open()
        for h in headers:
            file.write(h)
            file.write('\n')
        file.close()

    def write(self, lines):
        """lines -- string generator"""
        self._counter += 1
        file = self.open()
        for line in lines:
            file.write(line)
            file.write('\n')
        file.close()

        if self._counter >= self._nfreq:
            self._ifreq += 1
            self._counter = 0
            self.make_filename(self._ifreq)

    def make_filename(self, ifreq, use_axes=False, digit=5):
        # The .gz is not put if AxisWriter is used or compress=no
        ext = '.gz' if (self._use_zip and not(use_axes)) else ''
        num_fmts = '{:0' + str(digit) + '}'
        self._filename = self._base_fn + num_fmts.format(ifreq) + ext


class AxisWriter(Writer):
    """Called to write files if the calculator gives vectors (heat-flux)
    
    Used in FluxWriter.
    For one frame, writes one file per axis.
    
    Keyword arguments:
    filename -- string, will precede the name of written files
    frequency -- integer 
    compresslevel -- integer, ==0 when compress=no in .cfg file
    axes -- list of strings, each the name of an axis
    """

    def __init__(self, filename, frequency, axes, compresslevel=6):
        Writer.__init__(self, filename, frequency,
                        compresslevel, use_axes=True)

        self._axes = axes
        self._dim = len(axes)
        self.make_afnames()

    def open(self):
        # Overwriting open, this way each file is opened
        return map(self._open, self._afnames, ['ab']*self._dim)

    def close(self, file_):
        file_.close()

    def write_header(self, headers):
        files = self.open()
        for h in headers:
            for file_ in files:
                file_.write(h + '\n')
        map(self.close, files)

    def write(self, lines):
        """
        lines: a generator of list of strings.
               Each string is the line for one axis.
        """
        self._counter += 1
        files = self.open()
        for line in lines:
            for i, file_ in enumerate(files):
                file_.write(line[i] + '\n')
                #files[axis].write('\n')
        map(self.close, files)

        if self._counter >= self._nfreq:
            self._ifreq += 1
            self._counter = 0
            self.make_filename(self._ifreq, use_axes=True)
            self.make_afnames()

    def make_afnames(self):
        # Stores the filenames for each axis
        ext = '.gz' if self._use_zip else ''
        self._afnames = [self._filename + axis + ext for axis in self._axes]


"""
MultiXxWriter: choose the options for Current, Flux and NetCDFFluxWriter.
"""

class MultiCurrentWriter:

    def __init__(self, setting, decomp_list, target_anames, group_names):

        self.__atm_writer = CurrentWriter( setting, decomp_list,
                target_anames, revision='atm')
        self.__grp_writer = CurrentWriter( setting, decomp_list,
                group_names, revision='grp')
        self.__inn_writer = None
        self.__out_writer = None

        if setting.wurp.decomp_group_current:
            self.__inn_writer = CurrentWriter( setting, decomp_list,
                    group_names, revision='inn')
            self.__out_writer = CurrentWriter( setting, decomp_list,
                    group_names, revision='out')

    def write_header(self):

        self.__atm_writer.write_header()
        self.__grp_writer.write_header()

        if self.__inn_writer:
            self.__inn_writer.write_header()

        if self.__out_writer:
            self.__out_writer.write_header()

    def write(self, istep, key_to_acurs, key_to_gcurs,
            key_to_icurs=None, key_to_ocurs=None):

        self.__atm_writer.write(istep, key_to_acurs)
        self.__grp_writer.write(istep, key_to_gcurs)

        if self.__inn_writer:
            self.__inn_writer.write(istep, key_to_icurs)

        if self.__out_writer:
            self.__out_writer.write(istep, key_to_ocurs)


class MultiFluxWriter:

    def __init__(self, setting, decomp_list,
                 target_anames, group_names, gpair_table=None, axes=None):

        fmt = setting.output.format 
        grain = setting.wurp.flux_grain

        if (grain, fmt) == ('atom', 'ascii'):
            self.__atm_writer = FluxWriter( setting, decomp_list,
                    target_anames, axes, revision='atm' )
            self.__grp_writer = None

        elif (grain, fmt) == ('group', 'ascii'):
            self.__atm_writer = None
            self.__grp_writer = FluxWriter( setting, decomp_list,
                    group_names, axes, gpair_table, revision='grp' )

        elif (grain, fmt) == ('both', 'ascii'):
            self.__atm_writer = FluxWriter( setting, decomp_list,
                    target_anames, axes, revision='atm' )
            self.__grp_writer = FluxWriter( setting, decomp_list,
                    group_names, axes, gpair_table, revision='grp' )

        elif (grain, fmt) == ('atom', 'netcdf'):
            self.__atm_writer = NetCDFFluxWriter( setting, decomp_list,
                    target_anames, axes, revision='atm')
            self.__grp_writer = None

        elif (grain, fmt) == ('group', 'netcdf'):
            self.__atm_writer = None
            self.__grp_writer = NetCDFFluxWriter( setting, decomp_list,
                    group_names, gpair_table, axes, revision='grp')

        elif (grain, fmt) == ('both', 'netcdf'):
            self.__atm_writer = NetCDFFluxWriter( setting, decomp_list,
                    target_anames, axes, revision='atm')
            self.__grp_writer = NetCDFFluxWriter( setting, decomp_list,
                    group_names, gpair_table, axes, revision='grp')

        else:
            pass

    # elif method == 'energy-flux' and fmt=='netcdf':
        # obj = NetCDFFluxWriter(setting=setting, decomp_list, target_anames,
                # group_names=group_names, gpair_table=gpair_table)

        
    def write_header(self):
        if self.__atm_writer: self.__atm_writer.write_header()
        if self.__grp_writer: self.__grp_writer.write_header()

    def write(self, istep, flux_atm=None, flux_grp=None):
        if self.__atm_writer: self.__atm_writer.write(istep, flux_atm)
        if self.__grp_writer: self.__grp_writer.write(istep, flux_grp)

"""
Writers below parse the lines for files, given output of a calculator.
Writer or AxisWriter are then used to write the files in case of ascii.
NetCDFFluxWriter does this part by itself.
"""


class CurrentWriter:

    _flag_char = '%'

    def __init__(self, setting, decomp_list, names, title='', revision=''):
        self.__setting = setting
        self.__revision = revision
        self.__names = names
        self.__title = title

        self._time_fmt = '{:>15.3f} [ps]'
        self._data_fmt = ('{{name:>14s}}  '
               '{{xx:{fmt}}} {{xy:{fmt}}} {{xz:{fmt}}} '
               '{{yx:{fmt}}} {{yy:{fmt}}} {{yz:{fmt}}} '
               '{{zx:{fmt}}} {{zy:{fmt}}} {{zz:{fmt}}} ').format(fmt='012.7e')

        # Setup the object using setting
        filename = setting.output.filename[0]
        frequency = setting.output.frequency
        if setting.output.compress:
            compresslevel = 6
        else:
            compresslevel = 0

        prefix, ext = os.path.splitext(filename)
        fmt = '{prefix}_' + self.__revision + '{ext}'
        mod_fn = fmt.format(prefix=prefix, ext=ext)
        self.__key_to_writer = dict(
                total = Writer(mod_fn, frequency, compresslevel) )

        self.__decomp_list = decomp_list
        if setting.output.decomp:
            for key in decomp_list + ['kinetic']:
                key_fn = self.get_key_filename(key, filename)
                self.__key_to_writer[key] = Writer(
                        key_fn, frequency, compresslevel)

    def get_key_filename(self, key, base_filename):
        prefix, ext = os.path.splitext(base_filename)
        fmt = '{prefix}_' + self.__revision + '_{key}{ext}'
        return fmt.format(prefix=prefix, key=key, ext=ext)

    def get_header_format(self, title):
        yield self._flag_char + 'title  ' + title
        yield self._flag_char + 'format ' + 'time ' + self._time_fmt
        yield self._flag_char + 'format ' + 'data ' + self._data_fmt

    def format(self, time, current, names):
        yield self._flag_char + 'time' + self._time_fmt.format(time)
        yield self._flag_char + 'data'
        for name, cur in zip(names, current):
            yield self._data_fmt.format(name=name,
                    xx=cur[0,0], xy=cur[0,1], xz=cur[0,2],
                    yx=cur[1,0], yy=cur[1,1], yz=cur[1,2],
                    zx=cur[2,0], zy=cur[2,1], zz=cur[2,2] )

    def write_header(self):
        # headers
        for key, writer in self.__key_to_writer.items():
            writer.write_header(self.get_header_format(self.__title))

    def write(self, istep, results):

        if self.__setting.output.decomp:
            for key in self.__decomp_list + ['kinetic']:
                lines = self.format(istep, results[key], self.__names)
                self.__key_to_writer[key].write(lines)

        # if self.__setting.output.decomp:
        #     self.
        #     if self.__setting.output.decomp_bonded:
        #         bond_types = ['bond','angle','torsion','improper']
        #     else:
        #         bond_types = ['bonded']
        #         for key in 
        #             lines = self.format(time, results[key], results['names'])
        #             self.__key_to_writer[key].write(lines)

        # else:
        #     for key in ['bonded', 'coulomb', 'vdw', 'vel']:
        #         lines = self.format(time, results[key], results['names'])
        #         self.__key_to_writer[key].write(lines)

        total_current = results['kinetic']
        for key in self.__decomp_list:
            # total_current = total_current + results[key]
            total_current += results[key]

        lines = self.format(istep, total_current, self.__names)
        self.__key_to_writer['total'].write(lines)

    def write_all(self, time_itr, results_itr):
        for time, results in zip(time_itr, results_itr):

            if self.__setting.output.decomp:
                for key in self.__decomp_list + ['kinetic']:
                    lines = self.format(time, results[key], self.__names)
                    self.__key_to_writer[key].write(lines)

            # if self.__setting.output.decomp:
            #     self.
            #     if self.__setting.output.decomp_bonded:
            #         bond_types = ['bond','angle','torsion','improper']
            #     else:
            #         bond_types = ['bonded']
            #         for key in 
            #             lines = self.format(time, results[key], results['names'])
            #             self.__key_to_writer[key].write(lines)

            # else:
            #     for key in ['bonded', 'coulomb', 'vdw', 'vel']:
            #         lines = self.format(time, results[key], results['names'])
            #         self.__key_to_writer[key].write(lines)

            total_current = results['kinetic']
            for key in self.__decomp_list:
                # total_current = total_current + results[key]
                total_current += results[key]

            lines = self.format(time, total_current, names)
            self.__key_to_writer['total'].write(lines)

    def gen_decomp_keys(self, decomp_list):
        # Generate flatten decomposition key list.
        for key in decomp_list:
            if isinstance(key, tuple) or isinstance(key, list):
                yield key[0]
            else:
                yield key


class FluxWriter:

    _flag_char = '%'

    def __init__(self, setting, decomp_list, names, axes,
                 pair_table=None, title='', revision=''):
        self.__setting = setting    # setting object from .cfg file
        self.__revision = revision
        self.__title = title
        self.__names = names
        self.__pair_table = pair_table  # Interaction table
        self.__time_fmt = '{:>15.3f} [ps]'

        if axes is None:
            self.__axes = ['']
        else:
            self.__axes = axes  # List of strings: the name of the axes

        self.__dim = len(self.__axes)

        # Setup the object using setting

        filename = setting.output.filename[0]

        frequency = setting.output.frequency
        if setting.output.compress:
            compresslevel = 6
        else:
            compresslevel = 0
        prefix, ext = os.path.splitext(filename)
        fmt = '{prefix}_' + self.__revision + '{ext}'
        mod_fn = fmt.format(prefix=prefix, ext=ext)
        
        self.__writer = AxisWriter(mod_fn, frequency, self.__axes,
                                   compresslevel)
            
        # Defines format
        # Header columns:
        self.__fmt_fmt = '{:>5} {:>12}'.format('donor', 'acceptor')
        # Values lines format:
        self.__data_fmt = '{:>12s} {:>12s}' 
        # Stores the name of the forces if output has to be decomposed
        self.__decomps = ['total']
        if setting.output.decomp:
            self.__decomps.extend(decomp_list)

        for ptype in self.__decomps:
            self.__fmt_fmt += ' {:>16}'.format(ptype)
        
        self.__num_decomps = len(self.__decomps)
        self.__data_fmt += ' {:>16.8e}' * self.__num_decomps

        # make {gname : igrp} dictionary
        self.__name_to_idx = None
        if pair_table:
            self.__name_to_idx = {
                        name: i for i, name in enumerate(self.__names)
                        }

    def get_header_format(self, title):
        yield self._flag_char + 'title  ' + title
        yield self._flag_char + 'format ' + 'time ' + self.__time_fmt
        yield self._flag_char + 'label ' + self.__fmt_fmt

    def write_header(self):
        self.__writer.write_header(self.get_header_format(self.__title))

    def write(self, istep, key_to_fluxes):
    
        to_fluxes_ar = numpy.array(key_to_fluxes)
        if self.__pair_table is None:
            lines = self.format(istep, key_to_fluxes, self.__names)
        else:
            lines = self.format_by_pairs(
                    istep, key_to_fluxes, self.__names, self.__pair_table)

        self.__writer.write(lines)

    def ok_write(self, i, j):
        if i < j:
            return True
        else:
            return self.__setting.wurp.enable_inverse_pair

    def format(self, time, key_to_fluxes, names):
        """Format each line, returns 2d numpy array"""
        yield [self._flag_char + 'time' + self.__time_fmt.format(time)] * 3
        yield [self._flag_char + 'data'] * 3
        # for name, current in zip(names, current):

        for itar_1, name_i in enumerate(names):
            for jtar_1, name_j in enumerate(names):
                if not self.ok_write(itar_1, jtar_1): continue

                """fluxes = [None for k in self.__decomps]
                for i, pot_type in enumerate(self.__decomps):
                    fluxes[i] = key_to_fluxes[pot_type][itar_1, jtar_1]

                fluxes_ar = numpy.array(fluxes).transpose()
                fluxes_ar = fluxes_ar.reshape(self.__dim, -1)

                line = [self.__data_fmt.format(name_i, name_j, *k)
                        for k in fluxes_ar]
                """

                fluxes = numpy.zeros([self.__num_decomps, self.__dim])
                for i, pot_type in enumerate(self.__decomps):
                    fluxes[i] = key_to_fluxes[pot_type][itar_1, jtar_1]

                line = [self.__data_fmt.format(name_i, name_j, *fluxes[:,i])
                        for i, axis in enumerate(self.__axes)]
                yield line

    def format_by_pairs(self, time, key_to_fluxes, names, pair_table):
        yield [self._flag_char + 'time' + self.__time_fmt.format(time)] * 3
        yield [self._flag_char + 'data'] * 3
        # for name, current in zip(names, current):
        for name_i, names_j in pair_table:
            itar_1 = self.__name_to_idx[name_i]

            for name_j in names_j:
                jtar_1 = self.__name_to_idx[name_j]
                if not self.ok_write(itar_1, jtar_1): continue

                fluxes = numpy.zeros([self.__num_decomps, self.__dim])
                for i, pot_type in enumerate(self.__decomps):
                    fluxes[i] = key_to_fluxes[pot_type][itar_1, jtar_1]
                
                """
                fluxes is either a list of floats or of list.
                Each float corresponding to the contribution of the forces
                (total, vdw, coulomb, ...) to the flux (J).
                Or each list containing floats, coordinates of the flux vector,
                each list element correspond to the contribution of the forces.
                """

                line = [self.__data_fmt.format(name_i, name_j, *fluxes[:,i])
                        for i, axis in enumerate(self.__axes)]
                yield line

    def write_all(self, time_itr, results_itr):
        for time, results in zip(time_itr, results_itr):

            if self.__setting.output.decomp:
                for key in self.__decomp_list + ['kinetic']:
                    lines = self.format(time, results[key], self.__names)
                    self.__key_to_writer[key].write(lines)

            # if self.__setting.output.decomp:
            #     self.
            #     if self.__setting.output.decomp_bonded:
            #         bond_types = ['bond','angle','torsion','improper']
            #     else:
            #         bond_types = ['bonded']
            #         for key in 
            #             lines = self.format(time, results[key], results['names'])
            #             self.__key_to_writer[key].write(lines)

            # else:
            #     for key in ['bonded', 'coulomb', 'vdw', 'vel']:
            #         lines = self.format(time, results[key], results['names'])
            #         self.__key_to_writer[key].write(lines)

            total_current = results['kinetic']
            for key in self.__decomp_list:
                # total_current = total_current + results[key]
                total_current += results[key]

            lines = self.format(time, total_current, self.__names)
            self.__key_to_writer['total'].write(lines)

    def gen_decomp_keys(self, decomp_list):
        """Generate flatten decomposition key list."""
        for key in decomp_list:
            if isinstance(key, tuple) or isinstance(key, list):
                yield key[0]
            else:
                yield key


import netCDF4 as netcdf
class NetCDFFluxWriter:
    """

    For example:

    netcdf file_name {
    dimensions:
            nframe = UNLIMITED ; // (20 currently)
            ngroup_pair = 3 ;
            ncomponent = 9 ;
    variables:
            float time(nframe) ;
                    time:units = "picosecond" ;
            string donors(ngroup_pair) ;
            string acceptors(ngroup_pair) ;
            string components(ncomponent) ;
            float flux(nframe, ngroup_pair, ncomponent) ;
                    flux:units = "kcal/mol/fs" ;

    // global attributes:
                    :title = "" ;
                    :application = "the CURP program" ;
                    :program = "wurp" ;
                    :programVersion = "0.7" ;
                    :Convetsions = "CURP" ;
                    :ConvetsionVersion = "0.7" ;
    data:

     time = 0, 0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.1, 0.11, 
        0.12, 0.13, 0.14, 0.15, 0.16, 0.17, 0.18, 0.19 ;

     donors = "00001_ALA", "00001_ALA", "00002_ALA" ;

     acceptors = "00002_ALA", "00003_ALA", "00003_ALA" ;

     components = "total", "bond", "angle", "torsion", "improper", "coulomb14", 
        "vdw14", "coulomb", "vdw" ;

     flux =
      0.3240475, -0.02477073, 0.01304006, 0.08249603, 0.2595297, -0.06853036, 
        0.01592382, 0.05231608, -0.005957155,
        .
        .
        .,
    }
    """

    version = 0.8

    def __init__(self, setting, decomp_list, names,
                 gpair_table=None, axes=None, title='', revision=''):

        self.__ncfile = None
        self.__setting = setting
        self.__revision = revision
        self.__title = title
        self.__names = names
        self.__pair_table = gpair_table
        self.__fp = setting.output.filename[0]
        self.__axes = axes
        # self.complevel = complevel
        # if setting.output.compress:
            # self.complevel = 1
            # self.use_zlib  = True
        # else:
            # self.complevel = 0
            # self.use_zlib  = False

        self.setup_components(decomp_list)

        self.setup()

    def setup(self):

        fp = self.get_fp()
        title = self.__title

        self.__ngroup_pair = len(self.__names)**2

        ncfile = netcdf.Dataset(fp, clobber=True,
                mode='w', format='NETCDF3_64BIT')

        donors, acceptors = self.setup_gpairs(self.__names, self.__pair_table)

        # set global attributes
        ncfile.title             = title
        ncfile.application       = 'the CURP program'
        ncfile.program           = 'wurp'
        ncfile.programVersion    = str(self.version)
        ncfile.Convetsions       = 'CURP'
        ncfile.ConvetsionVersion = str(self.version)

        # create dimensions
        ncfile.createDimension('nframe', None)
        ncfile.createDimension('npair', self.__ngroup_pair)
        ncfile.createDimension('ncomponent' , len(self.__decomps))
        ncfile.createDimension('nchar', 20)
        # file.createDimensions('label', ?)

        # create variables
        # time
        nc_time = ncfile.createVariable('time', 'f4', ('nframe',))
        nc_time.units = 'picosecond'

        # donor and acceptor
        nc_don = ncfile.createVariable('donors',    'c', ('npair', 'nchar'))
        nc_acc = ncfile.createVariable('acceptors', 'c', ('npair', 'nchar'))

        # components
        nc_com = ncfile.createVariable('components','c',
                                      ('ncomponent','nchar'))

        """
        Differences whether the data format is
         - a float (energy-flux) or
         - a vector (heat-flux)
        """
        if self.__axes == None:
            # flux trajectory
            nc_flux = ncfile.createVariable('flux', 'f4',
                    ('nframe', 'npair', 'ncomponent'), )
            self.write = self.energy_write
        else:  
            # create axes dimension
            ncfile.createDimension('naxes', len(self.__axes))
            # flux trajectory
            nc_flux = ncfile.createVariable('flux', 'f4',
                    ('nframe', 'naxes', 'npair', 'ncomponent'), ) 
                    # zlib=self.use_zlib, complevel=self.complevel)
            self.write = self.heat_write

        nc_flux.units = 'kcal/mol/fs'

        # write group pairs
        for ipair_1, (don, acc) in enumerate(zip(donors, acceptors)):
            nc_don[ipair_1] = don.ljust(20)
            nc_acc[ipair_1] = acc.ljust(20)

        # write component names
        for icom_1, com in enumerate(self.__decomps):
            nc_com[icom_1] = com.ljust(20)

        ncfile.sync()
        ncfile.close()
        # self.__ncfile = ncfile

    def setup_gpairs(self, names, pair_table=None):

        name_to_idx = {name:i for i, name in enumerate(names)}
        ntarget = len(names)
        self.__npair_total = ntarget*ntarget

        # make mask indices and name pair list
        mask_indices = []
        donors = []
        acceptors = []
        if pair_table:
            for name_i, names_j in pair_table:
                itar_1 = name_to_idx[name_i]

                for name_j in names_j:
                    jtar_1 = name_to_idx[name_j]
                    if not self.ok_pair(itar_1, jtar_1): continue

                    ipair = ntarget*itar_1 + jtar_1
                    mask_indices += [ipair]
                    donors       += [name_i]
                    acceptors    += [name_j]

        else:
            for name_i in self.__names:
                itar_1 = name_to_idx[name_i]

                for name_j in self.__names:
                    jtar_1 = name_to_idx[name_j]
                    if not self.ok_pair(itar_1, jtar_1): continue

                    ipair = ntarget*itar_1 + jtar_1
                    mask_indices += [ipair]
                    donors       += [name_i]
                    acceptors    += [name_j]

        self.__mask_indices = mask_indices
        self.__ngroup_pair = len(self.__mask_indices)

        return donors, acceptors

    def setup_components(self, decomp_list):

        # Write components
        if self.__setting.output.decomp:
            self.__decomps = ['total'] + decomp_list
        else:
            self.__decomps = ['total']

    def ok_pair(self, i, j):
        if i < j:
            return True
        else:
            return self.__setting.wurp.enable_inverse_pair

    def write_header(self):
        pass

    def get_fp(self):
        prefix, ext = os.path.splitext(self.__fp)
        fmt = '{prefix}_' + self.__revision + '{ext}'
        return fmt.format(prefix=prefix, ext=ext)

    def energy_write(self, istp_1, key_to_fluxes):

        ncfile = self.open()

        # Write time
        dt = 0.01
        ncfile.variables['time'][istp_1] = istp_1*dt

        # Write flux
        nc_flux  = ncfile.variables['flux']

        flux = numpy.array([ key_to_fluxes[pot_type].ravel()
                for pot_type in self.__decomps ]).T

        if self.__mask_indices:
            nc_flux[istp_1] = flux[self.__mask_indices,:]
        else:
            nc_flux[istp_1] = flux
        
        self.close()

    def heat_write(self, istp_1, key_to_fluxes):

        ncfile = self.open()

        # Write time
        dt = 0.01
        ncfile.variables['time'][istp_1] = istp_1*dt

        # Write flux
        nc_flux  = ncfile.variables['flux']

        flux = numpy.array([ key_to_fluxes[pot_type]
                for pot_type in self.__decomps ]).T.reshape(
                                                    (len(self.__axes),
                                                     -1,
                                                     len(self.__decomps))
                                                    )

        if self.__mask_indices:
            nc_flux[istp_1] = flux[:,self.__mask_indices,:]
        else:
            nc_flux[istp_1] = flux
        
        self.close()
    def open(self):
        if self.__ncfile is None:
            ncfile = netcdf.Dataset(self.get_fp(), mode='r+')
            self.__ncfile = ncfile

        return self.__ncfile

    def close(self):
        if self.__ncfile:
            self.__ncfile.sync()
            self.__ncfile.close()
            self.__ncfile = None

